@@ -13,8 +13,10 @@
 #include "lc_previewwidget.h"
 
 #ifdef Q_OS_WIN
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
 #include <QtPlatformHeaders\QWindowsWindowFunctions>
 #endif
+#endif
 
 lcApplication* gApplication;
 
@@ -64,9 +66,13 @@ void lcPreferences::LoadDefaults()
 	mPreviewViewSphereSize = lcGetProfileInt(LC_PROFILE_PREVIEW_VIEW_SPHERE_SIZE);
 	mPreviewViewSphereLocation = static_cast<lcViewSphereLocation>(lcGetProfileInt(LC_PROFILE_PREVIEW_VIEW_SPHERE_LOCATION));
 	mDrawPreviewAxis = lcGetProfileInt(LC_PROFILE_PREVIEW_DRAW_AXES);
+	mStudCylinderColorEnabled = lcGetProfileInt(LC_PROFILE_STUD_CYLINDER_COLOR_ENABLED);
 	mStudCylinderColor = lcGetProfileInt(LC_PROFILE_STUD_CYLINDER_COLOR);
+	mPartEdgeColorEnabled = lcGetProfileInt(LC_PROFILE_PART_EDGE_COLOR_ENABLED);
 	mPartEdgeColor = lcGetProfileInt(LC_PROFILE_PART_EDGE_COLOR);
+	mBlackEdgeColorEnabled = lcGetProfileInt(LC_PROFILE_BLACK_EDGE_COLOR_ENABLED);
 	mBlackEdgeColor = lcGetProfileInt(LC_PROFILE_BLACK_EDGE_COLOR);
+	mDarkEdgeColorEnabled = lcGetProfileInt(LC_PROFILE_DARK_EDGE_COLOR_ENABLED);
 	mDarkEdgeColor = lcGetProfileInt(LC_PROFILE_DARK_EDGE_COLOR);
 	mPartEdgeContrast = lcGetProfileFloat(LC_PROFILE_PART_EDGE_CONTRAST);
 	mPartColorValueLDIndex = lcGetProfileFloat(LC_PROFILE_PART_COLOR_VALUE_LD_INDEX);
@@ -124,9 +130,13 @@ void lcPreferences::SaveDefaults()
 	lcSetProfileInt(LC_PROFILE_PREVIEW_VIEW_SPHERE_SIZE, mPreviewViewSphereSize);
 	lcSetProfileInt(LC_PROFILE_PREVIEW_VIEW_SPHERE_LOCATION, static_cast<int>(mPreviewViewSphereLocation));
 	lcSetProfileInt(LC_PROFILE_PREVIEW_DRAW_AXES, mDrawPreviewAxis);
+	lcSetProfileInt(LC_PROFILE_STUD_CYLINDER_COLOR_ENABLED, mStudCylinderColorEnabled);
 	lcSetProfileInt(LC_PROFILE_STUD_CYLINDER_COLOR, mStudCylinderColor);
+	lcSetProfileInt(LC_PROFILE_PART_EDGE_COLOR_ENABLED, mPartEdgeColorEnabled);
 	lcSetProfileInt(LC_PROFILE_PART_EDGE_COLOR, mPartEdgeColor);
+	lcSetProfileInt(LC_PROFILE_BLACK_EDGE_COLOR_ENABLED, mBlackEdgeColorEnabled);
 	lcSetProfileInt(LC_PROFILE_BLACK_EDGE_COLOR, mBlackEdgeColor);
+	lcSetProfileInt(LC_PROFILE_DARK_EDGE_COLOR_ENABLED, mDarkEdgeColorEnabled);
 	lcSetProfileInt(LC_PROFILE_DARK_EDGE_COLOR, mDarkEdgeColor);
 	lcSetProfileFloat(LC_PROFILE_PART_EDGE_CONTRAST, mPartEdgeContrast);
 	lcSetProfileFloat(LC_PROFILE_PART_COLOR_VALUE_LD_INDEX, mPartColorValueLDIndex);
@@ -284,7 +294,7 @@ void lcApplication::SetProject(Project* Project)
 	delete mProject;
 	mProject = Project;
 
-	Project->SetActiveModel(0);
+	Project->SetActiveModel(0, true);
 	lcGetPiecesLibrary()->RemoveTemporaryPieces();
 
 	if (mProject && !mProject->GetFileName().isEmpty() && mPreferences.mRestoreTabLayout)
@@ -373,9 +383,13 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 	Options.Viewpoint = lcViewpoint::Count;
 	Options.FadeStepsColor = Preferences.mFadeStepsColor;
 	Options.HighlightColor = Preferences.mHighlightNewPartsColor;
+	Options.StudCylinderColorEnabled = Preferences.mStudCylinderColorEnabled;
 	Options.StudCylinderColor = Preferences.mStudCylinderColor;
+	Options.PartEdgeColorEnabled = Preferences.mPartEdgeColorEnabled;
 	Options.PartEdgeColor = Preferences.mPartEdgeColor;
+	Options.BlackEdgeColorEnabled = Preferences.mBlackEdgeColorEnabled;
 	Options.BlackEdgeColor = Preferences.mBlackEdgeColor;
+	Options.DarkEdgeColorEnabled = Preferences.mDarkEdgeColorEnabled;
 	Options.DarkEdgeColor = Preferences.mDarkEdgeColor;
 	Options.PartEdgeContrast = Preferences.mPartEdgeContrast;
 	Options.PartColorValueLDIndex = Preferences.mPartColorValueLDIndex;
@@ -590,7 +604,7 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 		}
 		else if (Option == QLatin1String("--camera-angles"))
 		{
-			if ((Options.SetCameraAngles = ParseFloatArray(2, Options.CameraLatLon, true)) && (fabsf(Options.CameraLatLon[0]) > 360.0f || fabsf(Options.CameraLatLon[1]) > 360.0f))
+			if ((Options.SetCameraAngles = ParseFloatArray(2, Options.CameraLatLon.GetFloats(), true)) && (fabsf(Options.CameraLatLon[0]) > 360.0f || fabsf(Options.CameraLatLon[1]) > 360.0f))
 			{
 				Options.StdErr += tr("Invalid parameter value(s) specified for the '%1' option: limits are +/- 360.\n").arg(Option);
 				Options.ParseOK = false;
@@ -598,21 +612,32 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 		}
 		else if (Option == QLatin1String("--camera-position") || Option == QLatin1String("--camera-position-ldraw"))
 		{
-			if ((Options.SetCameraPosition = ParseFloatArray(9, Options.CameraPosition[0], true)))
+			if ((Options.SetCameraPosition = ParseFloatArray(9, Options.CameraPosition[0].GetFloats(), true)))
 			{
-				lcVector3 Front = Options.CameraPosition[1] - Options.CameraPosition[0];
-
-				if (Front.LengthSquared() < 1.0f || Options.CameraPosition[2].LengthSquared() < 1.0f || fabsf(lcDot(lcNormalize(Front), lcNormalize(Options.CameraPosition[2]))) > 0.99f)
-				{
-					Options.StdErr += tr("Invalid parameter value(s) specified for the '%1' option.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-				else if (Option == QLatin1String("--camera-position-ldraw"))
+				if (Option == QLatin1String("--camera-position-ldraw"))
 				{
 					Options.CameraPosition[0] = lcVector3LDrawToLeoCAD(Options.CameraPosition[0]);
 					Options.CameraPosition[1] = lcVector3LDrawToLeoCAD(Options.CameraPosition[1]);
 					Options.CameraPosition[2] = lcVector3LDrawToLeoCAD(Options.CameraPosition[2]);
 				}
+
+				lcVector3 Front = Options.CameraPosition[1] - Options.CameraPosition[0];
+
+				if (Front.LengthSquared() < 0.1f)
+				{
+					Options.StdErr += tr("The camera position is too close to the camera target of the '%1' option.\n").arg(Option);
+					Options.ParseOK = false;
+				}
+				else if (Options.CameraPosition[2].LengthSquared() < 0.1f)
+				{
+					Options.StdErr += tr("Invalid up vector length specified for the '%1' option.\n").arg(Option);
+					Options.ParseOK = false;
+				}
+				else if (fabsf(lcDot(lcNormalize(Front), lcNormalize(Options.CameraPosition[2]))) > 0.99f)
+				{
+					Options.StdErr += tr("The up vector is parallel to the front vector specified for the '%1' option.\n").arg(Option);
+					Options.ParseOK = false;
+				}
 			}
 		}
 		else if (Option == QLatin1String("--orthographic"))
@@ -621,82 +646,34 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 			Options.SetFoV = ParseFloat(Options.FoV, 1.0f, 180.0f);
 		else if (Option == QLatin1String("--zplanes"))
 		{
-			if ((Options.SetZPlanes = ParseFloatArray(2, Options.ZPlanes, false)) && (Options.ZPlanes[0] < 1.0 || Options.ZPlanes[0] >= Options.ZPlanes[1]))
+			if ((Options.SetZPlanes = ParseFloatArray(2, Options.ZPlanes.GetFloats(), false)) && (Options.ZPlanes[0] < 1.0 || Options.ZPlanes[0] >= Options.ZPlanes[1]))
 			{
 				Options.StdErr += tr("Invalid parameter value(s) specified for the '%1' option: requirements are: 1 <= <near> < <far>.\n").arg(Option);
 				Options.ParseOK = false;
 			}
 		}
+		else if (Option == QLatin1String("-nscc") || Option == QLatin1String("--disable-stud-cylinder-color"))
+			Options.StudCylinderColorEnabled = false;
 		else if (Option == QLatin1String("-scc") || Option == QLatin1String("--stud-cylinder-color"))
-		{
-			if (ParseColor(Options.StudCylinderColor))
-			{
-				if (!lcIsHighContrast(Options.StudStyle))
-				{
-					Options.StdErr += tr("High contrast stud style is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseColor(Options.StudCylinderColor);
+		else if (Option == QLatin1String("-nec") || Option == QLatin1String("--disable-edge-color"))
+			Options.PartEdgeColorEnabled = false;
 		else if (Option == QLatin1String("-ec") || Option == QLatin1String("--edge-color"))
-		{
-			if (ParseColor(Options.PartEdgeColor))
-			{
-				if (!lcIsHighContrast(Options.StudStyle))
-				{
-					Options.StdErr += tr("High contrast stud style is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseColor(Options.PartEdgeColor);
+		else if (Option == QLatin1String("-nbec") || Option == QLatin1String("--disable-black-edge-color"))
+			Options.BlackEdgeColorEnabled = false;
 		else if (Option == QLatin1String("-bec") || Option == QLatin1String("--black-edge-color"))
-		{
-			if (ParseColor(Options.BlackEdgeColor))
-			{
-				if (!lcIsHighContrast(Options.StudStyle))
-				{
-					Options.StdErr += tr("High contrast stud style is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseColor(Options.BlackEdgeColor);
+		else if (Option == QLatin1String("-ndec") || Option == QLatin1String("--disable-dark-edge-color"))
+			Options.DarkEdgeColorEnabled = false;
 		else if (Option == QLatin1String("-dec") || Option == QLatin1String("--dark-edge-color"))
-		{
-			if (ParseColor(Options.DarkEdgeColor))
-			{
-				if (!lcIsHighContrast(Options.StudStyle))
-				{
-					Options.StdErr += tr("High contrast stud style is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseColor(Options.DarkEdgeColor);
 		else if (Option == QLatin1String("-aec") || Option == QLatin1String("--automate-edge-color"))
-		{
 			Options.AutomateEdgeColor = true;
-		}
 		else if (Option == QLatin1String("-cc") || Option == QLatin1String("--color-contrast"))
-		{
-			if (ParseFloat(Options.PartEdgeContrast, 0.0f, 1.0f))
-			{
-				if (!Options.AutomateEdgeColor)
-				{
-					Options.StdErr += tr("Automate edge color is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseFloat(Options.PartEdgeContrast, 0.0f, 1.0f);
 		else if (Option == QLatin1String("-ldv") || Option == QLatin1String("--light-dark-value"))
-		{
-			if (ParseFloat(Options.PartColorValueLDIndex, 0.0f, 1.0f))
-			{
-				if (!Options.AutomateEdgeColor)
-				{
-					Options.StdErr += tr("Automate edge color is required for the '%1' option but is not enabled.\n").arg(Option);
-					Options.ParseOK = false;
-				}
-			}
-		}
+			ParseFloat(Options.PartColorValueLDIndex, 0.0f, 1.0f);
 		else if (Option == QLatin1String("--fade-steps"))
 			Options.FadeSteps = true;
 		else if (Option == QLatin1String("--no-fade-steps"))
@@ -805,9 +782,8 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 			Options.StdOut += tr("  -f, --from <step>: Set the first step to save pictures.\n");
 			Options.StdOut += tr("  -t, --to <step>: Set the last step to save pictures.\n");
 			Options.StdOut += tr("  -s, --submodel <submodel>: Set the active submodel.\n");
-			Options.StdOut += tr("  -c, --camera <camera>: Set the active camera.\n");
-			Options.StdOut += tr("  -ss, --stud-style <id>: Set the stud style 0=No style, 1=LDraw single wire, 2=LDraw double wire, 3=LDraw raised floating, 4=LDraw raised rounded, 5=LDraw subtle rounded, 6=LEGO no logo, 7=LEGO single wire.\n");
-			Options.StdOut += tr("  --viewpoint <front|back|left|right|top|bottom|home>: Set the viewpoint.\n");
+			Options.StdOut += tr("  -c, --camera <camera name>: Set the active camera.\n");
+			Options.StdOut += tr("  --viewpoint <viewpoint>: Set the view to one of the preset viewpoints (front, back, left, right, top, bottom, home).\n");
 			Options.StdOut += tr("  --camera-angles <latitude> <longitude>: Set the camera angles in degrees around the model.\n");
 			Options.StdOut += tr("  --camera-position <x> <y> <z> <tx> <ty> <tz> <ux> <uy> <uz>: Set the camera position, target and up vector.\n");
 			Options.StdOut += tr("  --camera-position-ldraw <x> <y> <z> <tx> <ty> <tz> <ux> <uy> <uz>: Set the camera position, target and up vector using LDraw coordinates.\n");
@@ -816,20 +792,25 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 			Options.StdOut += tr("  --zplanes <near> <far>: Set the near and far clipping planes used to render images (1 <= <near> < <far>).\n");
 			Options.StdOut += tr("  --fade-steps: Render parts from prior steps faded.\n");
 			Options.StdOut += tr("  --no-fade-steps: Do not render parts from prior steps faded.\n");
-			Options.StdOut += tr("  --fade-steps-color <rgba>: Renderinng color for prior step parts (#AARRGGBB).\n");
+			Options.StdOut += tr("  --fade-steps-color <#AARRGGBB>: Renderinng color for prior step parts.\n");
 			Options.StdOut += tr("  --highlight: Highlight parts in the steps they appear.\n");
 			Options.StdOut += tr("  --no-highlight: Do not highlight parts in the steps they appear.\n");
-			Options.StdOut += tr("  --highlight-color: Renderinng color for highlighted parts (#AARRGGBB).\n");
-			Options.StdOut += tr("  --shading <wireframe|flat|default|full>: Select shading mode for rendering.\n");
+			Options.StdOut += tr("  --highlight-color <#AARRGGBB>: Rendering color for highlighted parts.\n");
+			Options.StdOut += tr("  --shading <mode>: Select shading mode for rendering (wireframe, flat, default, full).\n");
 			Options.StdOut += tr("  --line-width <width>: Set the width of the edge lines.\n");
-			Options.StdOut += tr("  --aa-samples <count>: AntiAliasing sample size (1, 2, 4, or 8).\n");
+			Options.StdOut += tr("  --aa-samples <samples>: AntiAliasing sample size (1, 2, 4, or 8).\n");
+			Options.StdOut += tr("  -ss, --stud-style <style>: Set the stud style (0=No style, 1=LDraw single wire, 2=LDraw double wire, 3=LDraw raised floating, 4=LDraw raised rounded, 5=LDraw subtle rounded, 6=LEGO no logo, 7=LEGO single wire).\n");
+			Options.StdOut += tr("  -nscc, --disable-stud-cylinder-color: Disable high contrast stud cylinder color.\n");
 			Options.StdOut += tr("  -scc, --stud-cylinder-color <#AARRGGBB>: High contrast stud cylinder color.\n");
+			Options.StdOut += tr("  -nec, --disable-edge-color: Disable high contrast edge color.\n");
 			Options.StdOut += tr("  -ec, --edge-color <#AARRGGBB>: High contrast edge color.\n");
+			Options.StdOut += tr("  -nbec, --disable-black-edge-color: Disable high contrast edge color for black parts.\n");
 			Options.StdOut += tr("  -bec, --black-edge-color <#AARRGGBB>: High contrast edge color for black parts.\n");
+			Options.StdOut += tr("  -ndec, --disable-dark-edge-color: Disable high contrast edge color for dark color parts.\n");
 			Options.StdOut += tr("  -dec, --dark-edge-color <#AARRGGBB>: High contrast edge color for dark color parts.\n");
 			Options.StdOut += tr("  -aec, --automate-edge-color: Enable automatically adjusted edge colors.\n");
-			Options.StdOut += tr("  -cc, --color-contrast <float>: Color contrast value between 0.0 and 1.0.\n");
-			Options.StdOut += tr("  -ldv, --light-dark-value <float>: Light/Dark color value between 0.0 and 1.0.\n");
+			Options.StdOut += tr("  -cc, --color-contrast <contrast>: Color contrast value between 0.0 and 1.0.\n");
+			Options.StdOut += tr("  -ldv, --light-dark-value <value>: Light/Dark color value between 0.0 and 1.0.\n");
 			Options.StdOut += tr("  -obj, --export-wavefront <outfile.obj>: Export the model to Wavefront OBJ format.\n");
 			Options.StdOut += tr("  -3ds, --export-3ds <outfile.3ds>: Export the model to 3D Studio 3DS format.\n");
 			Options.StdOut += tr("  -dae, --export-collada <outfile.dae>: Export the model to COLLADA DAE format.\n");
@@ -852,6 +833,46 @@ lcCommandLineOptions lcApplication::ParseCommandLineOptions()
 		Options.StdErr += tr("High contrast stud and edge color settings are ignored when -aec or --automate-edge-color is set.\n");
 	}
 
+	if (!Options.AutomateEdgeColor && !Preferences.mAutomateEdgeColor)
+	{
+		QString const Message = tr("Automate edge color is required for the '%1' option but is not enabled.\n");
+
+		if (Options.PartEdgeContrast != Preferences.mPartEdgeContrast)
+			Options.StdErr += Message.arg(QString("--color-contrast %1").arg(Options.PartEdgeContrast));
+
+		if (Options.PartColorValueLDIndex != Preferences.mPartColorValueLDIndex)
+			Options.StdErr += Message.arg(QString("--light-dark-value %1").arg(Options.PartColorValueLDIndex));
+	}
+
+	if (!lcIsHighContrast(Options.StudStyle) && !lcIsHighContrast(static_cast<lcStudStyle>(lcGetProfileInt(LC_PROFILE_STUD_STYLE))))
+	{
+		QString const Message = tr("High contrast stud style is required for the '%1' option but is not enabled.\n");
+
+		if (Options.StudCylinderColorEnabled != Preferences.mStudCylinderColorEnabled)
+			Options.StdErr += Message.arg(QLatin1String("--disable-stud-cylinder-color"));
+
+		if (Options.StudCylinderColor != Preferences.mStudCylinderColor)
+			Options.StdErr += Message.arg(QString("--stud-cylinder-color %1").arg(Options.StudCylinderColor));
+
+		if (Options.PartEdgeColorEnabled != Preferences.mPartEdgeColorEnabled)
+			Options.StdErr += Message.arg(QLatin1String("--disable-edge-color"));
+
+		if (Options.PartEdgeColor != Preferences.mPartEdgeColor)
+			Options.StdErr += Message.arg(QString("--edge-color %1").arg(Options.PartEdgeColor));
+
+		if (Options.BlackEdgeColorEnabled != Preferences.mBlackEdgeColorEnabled)
+			Options.StdErr += Message.arg(QLatin1String("--disable-black-edge-color"));
+
+		if (Options.BlackEdgeColor != Preferences.mBlackEdgeColor)
+			Options.StdErr += Message.arg(QString("--black-edge-color %1").arg(Options.BlackEdgeColor));
+
+		if (Options.DarkEdgeColorEnabled != Preferences.mDarkEdgeColorEnabled)
+			Options.StdErr += Message.arg(QLatin1String("--disable-dark-edge-color"));
+
+		if (Options.DarkEdgeColor != Preferences.mDarkEdgeColor)
+			Options.StdErr += Message.arg(QString("--dark-edge-color %1").arg(Options.DarkEdgeColor));
+	}
+
 	if (!Options.CameraName.isEmpty())
 	{
 		if (Options.Viewpoint != lcViewpoint::Count)
@@ -953,15 +974,19 @@ lcStartupMode lcApplication::Initialize(const QList<QPair<QString, bool>>& Libra
 
 	mPreferences.mShadingMode = Options.ShadingMode;
 	mPreferences.mLineWidth = Options.LineWidth;
+	mPreferences.mStudCylinderColorEnabled = Options.StudCylinderColorEnabled;
 	mPreferences.mStudCylinderColor = Options.StudCylinderColor;
+	mPreferences.mPartEdgeColorEnabled = Options.PartEdgeColorEnabled;
 	mPreferences.mPartEdgeColor = Options.PartEdgeColor;
+	mPreferences.mBlackEdgeColorEnabled = Options.BlackEdgeColorEnabled;
 	mPreferences.mBlackEdgeColor = Options.BlackEdgeColor;
+	mPreferences.mDarkEdgeColorEnabled = Options.DarkEdgeColorEnabled;
 	mPreferences.mDarkEdgeColor = Options.DarkEdgeColor;
 	mPreferences.mPartEdgeContrast = Options.PartEdgeContrast;
 	mPreferences.mPartColorValueLDIndex = Options.PartColorValueLDIndex;
 	mPreferences.mAutomateEdgeColor = Options.AutomateEdgeColor;
 
-	lcGetPiecesLibrary()->SetStudStyle(Options.StudStyle, false);
+	lcGetPiecesLibrary()->SetStudStyle(Options.StudStyle, false, Options.StudCylinderColorEnabled);
 
 	if (!SaveAndExit)
 		gMainWindow->CreateWidgets();
@@ -997,7 +1022,7 @@ lcStartupMode lcApplication::Initialize(const QList<QPair<QString, bool>>& Libra
 	if (ProjectLoaded)
 	{
 		if (!Options.ModelName.isEmpty())
-			mProject->SetActiveModel(Options.ModelName);
+			mProject->SetActiveModel(Options.ModelName, true);
 
 		std::unique_ptr<lcView> ActiveView;
 
@@ -1227,7 +1252,9 @@ lcStartupMode lcApplication::Initialize(const QList<QPair<QString, bool>>& Libra
 		gMainWindow->show();
 
 #ifdef Q_OS_WIN
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
 		QWindowsWindowFunctions::setHasBorderInFullScreen(gMainWindow->windowHandle(), true);
+#endif
 #endif
 	}
 
@@ -1285,16 +1312,22 @@ void lcApplication::ShowPreferencesDialog()
 
 	bool LanguageChanged = Options.Language != lcGetProfileString(LC_PROFILE_LANGUAGE);
 	bool LibraryChanged = Options.LibraryPath != lcGetProfileString(LC_PROFILE_PARTS_LIBRARY);
-	bool ColorsChanged = Options.ColorConfigPath != lcGetProfileString(LC_PROFILE_COLOR_CONFIG);
+	bool ColorConfigChanged = Options.ColorConfigPath != lcGetProfileString(LC_PROFILE_COLOR_CONFIG);
 	bool AAChanged = CurrentAASamples != Options.AASamples;
 	bool StudStyleChanged = CurrentStudStyle != Options.StudStyle;
-	bool AutomateEdgeColorChanged = Options.Preferences.mAutomateEdgeColor != mPreferences.mAutomateEdgeColor;
-	AutomateEdgeColorChanged |= Options.Preferences.mStudCylinderColor != mPreferences.mStudCylinderColor;
-	AutomateEdgeColorChanged |= Options.Preferences.mPartEdgeColor != mPreferences.mPartEdgeColor;
-	AutomateEdgeColorChanged |= Options.Preferences.mBlackEdgeColor != mPreferences.mBlackEdgeColor;
-	AutomateEdgeColorChanged |= Options.Preferences.mDarkEdgeColor != mPreferences.mDarkEdgeColor;
-	AutomateEdgeColorChanged |= Options.Preferences.mPartEdgeContrast != mPreferences.mPartEdgeContrast;
-	AutomateEdgeColorChanged |= Options.Preferences.mPartColorValueLDIndex != mPreferences.mPartColorValueLDIndex;
+	bool StudCylinderColorEnabledChanged = Options.Preferences.mStudCylinderColorEnabled != mPreferences.mStudCylinderColorEnabled;
+	bool EdgeColorSettingsChanged = Options.Preferences.mAutomateEdgeColor != mPreferences.mAutomateEdgeColor;
+	EdgeColorSettingsChanged |= Options.Preferences.mPartEdgeContrast != mPreferences.mPartEdgeContrast;
+	EdgeColorSettingsChanged |= Options.Preferences.mPartColorValueLDIndex != mPreferences.mPartColorValueLDIndex;
+	EdgeColorSettingsChanged |= Options.Preferences.mStudCylinderColorEnabled != mPreferences.mStudCylinderColorEnabled;
+	EdgeColorSettingsChanged |= Options.Preferences.mStudCylinderColor != mPreferences.mStudCylinderColor;
+	EdgeColorSettingsChanged |= Options.Preferences.mPartEdgeColorEnabled != mPreferences.mPartEdgeColorEnabled;
+	EdgeColorSettingsChanged |= Options.Preferences.mPartEdgeColor != mPreferences.mPartEdgeColor;
+	EdgeColorSettingsChanged |= Options.Preferences.mBlackEdgeColorEnabled != mPreferences.mBlackEdgeColorEnabled;
+	EdgeColorSettingsChanged |= Options.Preferences.mBlackEdgeColor != mPreferences.mBlackEdgeColor;
+	EdgeColorSettingsChanged |= Options.Preferences.mDarkEdgeColorEnabled != mPreferences.mDarkEdgeColorEnabled;
+	EdgeColorSettingsChanged |= Options.Preferences.mDarkEdgeColor != mPreferences.mDarkEdgeColor;
+
 
 	mPreferences = Options.Preferences;
 
@@ -1351,15 +1384,16 @@ void lcApplication::ShowPreferencesDialog()
 		}
 	}
 
-	if (StudStyleChanged)
+	if (StudStyleChanged || StudCylinderColorEnabledChanged)
 	{
 		lcSetProfileInt(LC_PROFILE_STUD_STYLE, static_cast<int>(Options.StudStyle));
-		lcGetPiecesLibrary()->SetStudStyle(Options.StudStyle, true);
+		lcGetPiecesLibrary()->SetStudStyle(Options.StudStyle, true, Options.Preferences.mStudCylinderColorEnabled);
 	}
-	else if (ColorsChanged || AutomateEdgeColorChanged)
+	else if (ColorConfigChanged || EdgeColorSettingsChanged)
 	{
-		if (ColorsChanged)
+		if (ColorConfigChanged)
 			lcSetProfileString(LC_PROFILE_COLOR_CONFIG, Options.ColorConfigPath);
+
 		lcGetPiecesLibrary()->LoadColors();
 	}
 
