@@ -1,17 +1,19 @@
 #pragma once
 
-#include "lc_array.h"
 #include "lc_math.h"
 
 #define LC_MAX_COLOR_NAME 64
 #define LC_COLOR_DIRECT 0x80000000
 #define LC_COLOR_NOCOLOR 0xffffffff
+#define LC_STUD_CYLINDER_COLOR_CODE 4242
 
 struct lcColor
 {
 	quint32 Code;
 	int Group;
 	bool Translucent = false;
+	bool Chrome = false;
+	bool Rubber = false;
 	bool Adjusted = false;
 	lcVector4 Value;
 	lcVector4 Edge;
@@ -63,3 +65,16 @@ inline bool lcIsColorTranslucent(size_t ColorIndex)
 {
 	return gColorList[ColorIndex].Translucent;
 }
+
+inline bool lcIsColorChrome(size_t ColorIndex)
+{
+	return gColorList[ColorIndex].Chrome;
+}
+
+inline bool lcIsColorRubber(size_t ColorIndex)
+{
+	return gColorList[ColorIndex].Rubber;
+}
+
+QString lcGetColorToolTip(int ColorIndex);
+void lcDrawNoColorRect(QPainter& Painter, const QRect& Rect);
