@@ -108,13 +108,28 @@ const lcCommand gCommands[] =
 		QT_TRANSLATE_NOOP("Status", "Export the current model in Wavefront OBJ format"),
 		""
 	},
-	// LC_FILE_RENDER
+	// LC_FILE_RENDER_POVRAY
 	{
-		QT_TRANSLATE_NOOP("Action", "File.Render"),
-		QT_TRANSLATE_NOOP("Menu", "&Render..."),
+		QT_TRANSLATE_NOOP("Action", "File.Render.POVRay"),
+		QT_TRANSLATE_NOOP("Menu", "&POV-Ray Render..."),
 		QT_TRANSLATE_NOOP("Status", "Render the current model using POV-Ray"),
 		""
 	},
+	// LC_FILE_RENDER_BLENDER
+	{
+		QT_TRANSLATE_NOOP("Action", "File.Render.Blender"),
+		QT_TRANSLATE_NOOP("Menu", "&Blender Render..."),
+		QT_TRANSLATE_NOOP("Status", "Render the current model using Blender"),
+		""
+	},
+
+	// LC_FILE_RENDER_OPEN_IN_BLENDER
+	{
+		QT_TRANSLATE_NOOP("Action", "File.Render.OpenInBlender"),
+		QT_TRANSLATE_NOOP("Menu", "&Open In Blender..."),
+		QT_TRANSLATE_NOOP("Status", "Open the current model in Blender"),
+		""
+	},
 	// LC_FILE_INSTRUCTIONS
 	{
 		QT_TRANSLATE_NOOP("Action", "File.Instructions"),
@@ -318,46 +333,46 @@ const lcCommand gCommands[] =
 		QT_TRANSLATE_NOOP("Status", "Select all pieces of the same type and color"),
 		""
 	},
-    // LC_EDIT_TRANSFORM_RELATIVE
-    {
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformRelative"),
-        QT_TRANSLATE_NOOP("Menu", "Relative Transforms"),
-        QT_TRANSLATE_NOOP("Status", "Move and rotate objects relative to the one that has focus"),
-        ""
-    },
-    // LC_EDIT_TRANSFORM_ABSOLUTE
-    {
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformAbsolute"),
-        QT_TRANSLATE_NOOP("Menu", "Absolute Transforms"),
-        QT_TRANSLATE_NOOP("Status", "Move and rotate objects in absolute coordinates"),
-        ""
-    },
-    // LC_EDIT_TRANSFORM_TOGGLE_RELATIVE
-    {
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformToggleRelative"),
-        QT_TRANSLATE_NOOP("Menu", "Toggle Relative Transforms"),
-        QT_TRANSLATE_NOOP("Status", "Toggle moving and rotating objects relative to the one that has focus"),
+	// LC_EDIT_TRANSFORM_RELATIVE
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformRelative"),
+		QT_TRANSLATE_NOOP("Menu", "Relative Transforms"),
+		QT_TRANSLATE_NOOP("Status", "Move and rotate objects relative to the one that has focus"),
 		""
 	},
-    // LC_EDIT_TRANSFORM_SEPARATELY
-    {
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformSeparately"),
-        QT_TRANSLATE_NOOP("Menu", "Rotate Separately"),
-        QT_TRANSLATE_NOOP("Status", "Rotate selected pieces separately"),
-        ""
-    },
-    // LC_EDIT_TRANSFORM_TOGETHER
-    {
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformTogether"),
-        QT_TRANSLATE_NOOP("Menu", "Rotate Together"),
-        QT_TRANSLATE_NOOP("Status", "Rotate selected pieces together"),
-        ""
-    },
-    // LC_EDIT_TRANSFORM_TOGGLE_SEPARATE
+	// LC_EDIT_TRANSFORM_ABSOLUTE
 	{
-        QT_TRANSLATE_NOOP("Action", "Edit.TransformToggleSeparate"),
-        QT_TRANSLATE_NOOP("Menu", "Toggle Separate Transforms"),
-        QT_TRANSLATE_NOOP("Status", "Toggle rotating selected pieces separately"),
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformAbsolute"),
+		QT_TRANSLATE_NOOP("Menu", "Absolute Transforms"),
+		QT_TRANSLATE_NOOP("Status", "Move and rotate objects in absolute coordinates"),
+		""
+	},
+	// LC_EDIT_TRANSFORM_TOGGLE_RELATIVE
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformToggleRelative"),
+		QT_TRANSLATE_NOOP("Menu", "Toggle Relative Transforms"),
+		QT_TRANSLATE_NOOP("Status", "Toggle moving and rotating objects relative to the one that has focus"),
+		""
+	},
+	// LC_EDIT_TRANSFORM_SEPARATELY
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformSeparately"),
+		QT_TRANSLATE_NOOP("Menu", "Rotate Separately"),
+		QT_TRANSLATE_NOOP("Status", "Rotate selected pieces separately"),
+		""
+	},
+	// LC_EDIT_TRANSFORM_TOGETHER
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformTogether"),
+		QT_TRANSLATE_NOOP("Menu", "Rotate Together"),
+		QT_TRANSLATE_NOOP("Status", "Rotate selected pieces together"),
+		""
+	},
+	// LC_EDIT_TRANSFORM_TOGGLE_SEPARATE
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.TransformToggleSeparate"),
+		QT_TRANSLATE_NOOP("Menu", "Toggle Separate Transforms"),
+		QT_TRANSLATE_NOOP("Status", "Toggle rotating selected pieces separately"),
 		""
 	},
 	// LC_EDIT_SNAP_MOVE_TOGGLE
@@ -626,18 +641,32 @@ const lcCommand gCommands[] =
 		QT_TRANSLATE_NOOP("Status", "Add new pieces to the model"),
 		""
 	},
-	// LC_EDIT_ACTION_LIGHT
+	// LC_EDIT_ACTION_POINT_LIGHT
 	{
-		QT_TRANSLATE_NOOP("Action", "Edit.Tool.Light"),
-		QT_TRANSLATE_NOOP("Menu", "Light"),
-		QT_TRANSLATE_NOOP("Status", "Add new omni light sources to the model"),
+		QT_TRANSLATE_NOOP("Action", "Edit.Tool.PointLight"),
+		QT_TRANSLATE_NOOP("Menu", "Point Light"),
+		QT_TRANSLATE_NOOP("Status", "Add new point light sources to the model"),
 		""
 	},
 	// LC_EDIT_ACTION_SPOTLIGHT
 	{
-		QT_TRANSLATE_NOOP("Action", "Edit.Tool.Spotlight"),
-		QT_TRANSLATE_NOOP("Menu", "Spotlight"),
-		QT_TRANSLATE_NOOP("Status", "Add new spotlights to the model"),
+		QT_TRANSLATE_NOOP("Action", "Edit.Tool.SpotLight"),
+		QT_TRANSLATE_NOOP("Menu", "Spot Light"),
+		QT_TRANSLATE_NOOP("Status", "Add new spot lights to the model"),
+		""
+	},
+	// LC_EDIT_ACTION_DIRECTIONAL_LIGHT
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.Tool.DirectionalLight"),
+		QT_TRANSLATE_NOOP("Menu", "Directional Light"),
+		QT_TRANSLATE_NOOP("Status", "Add new omnidirectional light sources to the model"),
+		""
+	},
+	// LC_EDIT_ACTION_AREA_LIGHT
+	{
+		QT_TRANSLATE_NOOP("Action", "Edit.Tool.AreaLight"),
+		QT_TRANSLATE_NOOP("Menu", "Area Light"),
+		QT_TRANSLATE_NOOP("Status", "Add new area light sources to the model"),
 		""
 	},
 	// LC_EDIT_ACTION_CAMERA
@@ -1249,6 +1278,27 @@ const lcCommand gCommands[] =
 		QT_TRANSLATE_NOOP("Status", "Remove the selected control point"),
 		""
 	},
+	// LC_PIECE_TRAIN_TRACK_FOCUS_NEXT
+	{
+		QT_TRANSLATE_NOOP("Action", "Piece.TrainTrack.FocusNext"),
+		QT_TRANSLATE_NOOP("Menu", "Focus Next Connection"),
+		QT_TRANSLATE_NOOP("Status", "Focus the next connection of the currently selected track"),
+		""
+	},
+	// LC_PIECE_TRAIN_TRACK_FOCUS_PREVIOUS
+	{
+		QT_TRANSLATE_NOOP("Action", "Piece.TrainTrack.FocusPrevious"),
+		QT_TRANSLATE_NOOP("Menu", "Focus Previous Connection"),
+		QT_TRANSLATE_NOOP("Status", "Focus the previous connection of the currently selected track"),
+		""
+	},
+	// LC_PIECE_TRAIN_TRACK_ROTATE
+	{
+		QT_TRANSLATE_NOOP("Action", "Piece.TrainTrack.Rotate"),
+		QT_TRANSLATE_NOOP("Menu", "Rotate Train Track"),
+		QT_TRANSLATE_NOOP("Status", "Rotate the focused train track arount its last connection"),
+		""
+	},
 	// LC_PIECE_MOVE_PLUSX
 	{
 		QT_TRANSLATE_NOOP("Action", "Piece.Move.PlusX"),
@@ -1347,7 +1397,7 @@ const lcCommand gCommands[] =
 		QT_TRANSLATE_NOOP("Status", "Make copies of the selected pieces"),
 		""
 	},
-	// 	LC_PIECE_VIEW_SELECTED_MODEL
+	//  LC_PIECE_VIEW_SELECTED_MODEL
 	{
 		QT_TRANSLATE_NOOP("Action", "Piece.ViewSelectedModel"),
 		QT_TRANSLATE_NOOP("Menu", "Open Selected Model"),
@@ -1843,21 +1893,23 @@ LC_ARRAY_SIZE_CHECK(gCommands, LC_NUM_COMMANDS);
 
 const char* gToolNames[] =
 {
-    QT_TRANSLATE_NOOP("Mouse", "NewPiece"),      // lcTool::Insert
-    QT_TRANSLATE_NOOP("Mouse", "NewPointLight"), // lcTool::Light
-    QT_TRANSLATE_NOOP("Mouse", "NewSpotLight"),  // lcTool::SpotLight
-    QT_TRANSLATE_NOOP("Mouse", "NewCamera"),     // lcTool::Camera
-    QT_TRANSLATE_NOOP("Mouse", "Select"),        // lcTool::Select
-    QT_TRANSLATE_NOOP("Mouse", "Move"),          // lcTool::Move
-    QT_TRANSLATE_NOOP("Mouse", "Rotate"),        // lcTool::Rotate
-    QT_TRANSLATE_NOOP("Mouse", "Delete"),        // lcTool::Eraser
-    QT_TRANSLATE_NOOP("Mouse", "Paint"),         // lcTool::Paint
-    QT_TRANSLATE_NOOP("Mouse", "ColorPicker"),   // lcTool::ColorPicker
-    QT_TRANSLATE_NOOP("Mouse", "Zoom"),          // lcTool::Zoom
-    QT_TRANSLATE_NOOP("Mouse", "Pan"),           // lcTool::Pan
-    QT_TRANSLATE_NOOP("Mouse", "Orbit"),         // lcTool::RotateView
-    QT_TRANSLATE_NOOP("Mouse", "Roll"),          // lcTool::Roll
-    QT_TRANSLATE_NOOP("Mouse", "ZoomRegion")     // lcTool::ZoomRegion
+	QT_TRANSLATE_NOOP("Mouse", "NewPiece"),            // lcTool::Insert
+	QT_TRANSLATE_NOOP("Mouse", "NewPointLight"),       // lcTool::PointLight
+	QT_TRANSLATE_NOOP("Mouse", "NewSpotLight"),        // lcTool::SpotLight
+	QT_TRANSLATE_NOOP("Mouse", "NewDirectionalLight"), // lcTool::DirectionalLight
+	QT_TRANSLATE_NOOP("Mouse", "NewAreaLight"),        // lcTool::AreaLight
+	QT_TRANSLATE_NOOP("Mouse", "NewCamera"),           // lcTool::Camera
+	QT_TRANSLATE_NOOP("Mouse", "Select"),              // lcTool::Select
+	QT_TRANSLATE_NOOP("Mouse", "Move"),                // lcTool::Move
+	QT_TRANSLATE_NOOP("Mouse", "Rotate"),              // lcTool::Rotate
+	QT_TRANSLATE_NOOP("Mouse", "Delete"),              // lcTool::Eraser
+	QT_TRANSLATE_NOOP("Mouse", "Paint"),               // lcTool::Paint
+	QT_TRANSLATE_NOOP("Mouse", "ColorPicker"),         // lcTool::ColorPicker
+	QT_TRANSLATE_NOOP("Mouse", "Zoom"),                // lcTool::Zoom
+	QT_TRANSLATE_NOOP("Mouse", "Pan"),                 // lcTool::Pan
+	QT_TRANSLATE_NOOP("Mouse", "Orbit"),               // lcTool::RotateView
+	QT_TRANSLATE_NOOP("Mouse", "Roll"),                // lcTool::Roll
+	QT_TRANSLATE_NOOP("Mouse", "ZoomRegion")           // lcTool::ZoomRegion
 };
 
 LC_ARRAY_SIZE_CHECK(gToolNames, lcTool::Count);
