@@ -1344,7 +1344,7 @@ void lcContext::FlushState()
 			if (mWorldMatrixDirty)
 			{
 				if (Program.WorldMatrixLocation != -1)
-					glUniformMatrix4fv(Program.WorldMatrixLocation, 1, false, mWorldMatrix);
+					glUniformMatrix4fv(Program.WorldMatrixLocation, 1, false, mWorldMatrix.GetFloats());
 			}
 
 			if (mViewMatrixDirty)
@@ -1355,14 +1355,14 @@ void lcContext::FlushState()
 				if (Program.LightPositionLocation != -1)
 				{
 					lcVector3 LightPosition = ViewPosition + lcMul30(lcVector3(300.0f, 300.0f, 0.0f), InverseViewMatrix);
-					glUniform3fv(Program.LightPositionLocation, 1, LightPosition);
+					glUniform3fv(Program.LightPositionLocation, 1, LightPosition.GetFloats());
 				}
 
 				if (Program.EyePositionLocation != -1)
-					glUniform3fv(Program.EyePositionLocation, 1, ViewPosition);
+					glUniform3fv(Program.EyePositionLocation, 1, ViewPosition.GetFloats());
 			}
 
-			glUniformMatrix4fv(Program.WorldViewProjectionMatrixLocation, 1, false, lcMul(mWorldMatrix, mViewProjectionMatrix));
+			glUniformMatrix4fv(Program.WorldViewProjectionMatrixLocation, 1, false, lcMul(mWorldMatrix, mViewProjectionMatrix).GetFloats());
 			mWorldMatrixDirty = false;
 			mViewMatrixDirty = false;
 			mProjectionMatrixDirty = false;
@@ -1370,20 +1370,20 @@ void lcContext::FlushState()
 
 		if (mColorDirty && Program.MaterialColorLocation != -1)
 		{
-			glUniform4fv(Program.MaterialColorLocation, 1, mColor);
+			glUniform4fv(Program.MaterialColorLocation, 1, mColor.GetFloats());
 			mColorDirty = false;
 		}
 
 		if (mHighlightParamsDirty && Program.HighlightParamsLocation != -1)
 		{
-			glUniform4fv(Program.HighlightParamsLocation, 4, mHighlightParams[0]);
+			glUniform4fv(Program.HighlightParamsLocation, 4, mHighlightParams[0].GetFloats());
 			mHighlightParamsDirty = false;
 		}
 	}
 	else
 	{
 #if LC_FIXED_FUNCTION
-		glColor4fv(mColor);
+		glColor4fv(mColor.GetFloats());
 
 		if (mWorldMatrixDirty || mViewMatrixDirty)
 		{
@@ -1393,7 +1393,7 @@ void lcContext::FlushState()
 				mMatrixMode = GL_MODELVIEW;
 			}
 
-			glLoadMatrixf(lcMul(mWorldMatrix, mViewMatrix));
+			glLoadMatrixf(lcMul(mWorldMatrix, mViewMatrix).GetFloats());
 			mWorldMatrixDirty = false;
 			mViewMatrixDirty = false;
 		}
@@ -1406,7 +1406,7 @@ void lcContext::FlushState()
 				mMatrixMode = GL_PROJECTION;
 			}
 
-			glLoadMatrixf(mProjectionMatrix);
+			glLoadMatrixf(mProjectionMatrix.GetFloats());
 			mProjectionMatrixDirty = false;
 		}
 #endif
