@@ -1,6 +1,5 @@
 #pragma once
 
-#include "lc_array.h"
 #include "lc_math.h"
 #include "lc_colors.h"
 #include "lc_mesh.h"
@@ -132,6 +131,11 @@ public:
 	void SetGLContext(QOpenGLContext* GLContext, QOpenGLWidget* Widget);
 	void SetOffscreenContext();
 
+	bool IsOffscreen() const
+	{
+		return mWidget == nullptr;
+	}
+
 	void ClearColorAndDepth(const lcVector4& ClearColor);
 	void ClearDepth();
 
