@@ -5,9 +5,13 @@ lcAutomateEdgeColorDialog::lcAutomateEdgeColorDialog(QWidget* Parent, bool ShowH
 	:QDialog(Parent)
 {
 	const lcPreferences& Preferences = lcGetPreferences();
+	mStudCylinderColorEnabled = Preferences.mStudCylinderColorEnabled;
 	mStudCylinderColor = Preferences.mStudCylinderColor;
+	mPartEdgeColorEnabled = Preferences.mPartEdgeColorEnabled;
 	mPartEdgeColor = Preferences.mPartEdgeColor;
+	mBlackEdgeColorEnabled = Preferences.mBlackEdgeColorEnabled;
 	mBlackEdgeColor = Preferences.mBlackEdgeColor;
+	mDarkEdgeColorEnabled = Preferences.mDarkEdgeColorEnabled;
 	mDarkEdgeColor = Preferences.mDarkEdgeColor;
 	mPartEdgeContrast = Preferences.mPartEdgeContrast;
 	mPartColorValueLDIndex = Preferences.mPartColorValueLDIndex;
@@ -80,55 +84,75 @@ lcAutomateEdgeColorDialog::lcAutomateEdgeColorDialog(QWidget* Parent, bool ShowH
 		Button->setToolTip(ButtonColor.name().toUpper());
 	};
 
-	QLabel* StudCylinderColorLabel = new QLabel(tr("Stud Cylinder Color:"), this);
+	StudCylinderColorEnabledBox = new QCheckBox(tr("Stud Cylinder Color:"), this);
+	StudCylinderColorEnabledBox->setChecked(mStudCylinderColorEnabled);
+	connect(StudCylinderColorEnabledBox, SIGNAL(clicked()), this, SLOT(ColorCheckBoxClicked()));
+
 	StudCylinderColorButton = new QToolButton(this);
+	StudCylinderColorButton->setEnabled(mStudCylinderColorEnabled);
 	SetButtonPixmap(mStudCylinderColor, StudCylinderColorButton);
 	connect(StudCylinderColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
 
 	ResetStudCylinderColorButton = new QToolButton(this);
-	ResetStudCylinderColorButton->setText(tr("Reset"));;
+	ResetStudCylinderColorButton->setText(tr("Reset"));
+	ResetStudCylinderColorButton->setEnabled(mStudCylinderColorEnabled);
 	connect(ResetStudCylinderColorButton, SIGNAL(clicked()), this, SLOT(ResetColorButtonClicked()));
 
-	HighContrastColorLayout->addWidget(StudCylinderColorLabel,0,0);
+	HighContrastColorLayout->addWidget(StudCylinderColorEnabledBox,0,0);
 	HighContrastColorLayout->addWidget(StudCylinderColorButton,0,1);
 	HighContrastColorLayout->addWidget(ResetStudCylinderColorButton,0,2);
 
-	QLabel* PartEdgeColorLabel = new QLabel(tr("Parts Edge Color:"), this);
+	PartEdgeColorEnabledBox = new QCheckBox(tr("Parts Edge Color:"), this);
+	PartEdgeColorEnabledBox->setChecked(mPartEdgeColorEnabled);
+	connect(PartEdgeColorEnabledBox, SIGNAL(clicked()), this, SLOT(ColorCheckBoxClicked()));
+
 	PartEdgeColorButton = new QToolButton(this);
+	PartEdgeColorButton->setEnabled(mPartEdgeColorEnabled);
 	SetButtonPixmap(mPartEdgeColor, PartEdgeColorButton);
 	connect(PartEdgeColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
 
 	ResetPartEdgeColorButton = new QToolButton(this);
 	ResetPartEdgeColorButton->setText(tr("Reset"));
+	ResetPartEdgeColorButton->setEnabled(mPartEdgeColorEnabled);
 	connect(ResetPartEdgeColorButton, SIGNAL(clicked()), this, SLOT(ResetColorButtonClicked()));
 
-	HighContrastColorLayout->addWidget(PartEdgeColorLabel,1,0);
+	HighContrastColorLayout->addWidget(PartEdgeColorEnabledBox,1,0);
 	HighContrastColorLayout->addWidget(PartEdgeColorButton,1,1);
 	HighContrastColorLayout->addWidget(ResetPartEdgeColorButton,1,2);
 
-	QLabel* BlackEdgeColorLabel = new QLabel(tr("Black Parts Edge Color:"), this);
+	BlackEdgeColorEnabledBox = new QCheckBox(tr("Black Parts Edge Color:"), this);
+	BlackEdgeColorEnabledBox->setChecked(mBlackEdgeColorEnabled);
+	connect(BlackEdgeColorEnabledBox, SIGNAL(clicked()), this, SLOT(ColorCheckBoxClicked()));
+
 	BlackEdgeColorButton = new QToolButton(this);
+	BlackEdgeColorButton->setEnabled(mBlackEdgeColorEnabled);
 	SetButtonPixmap(mBlackEdgeColor, BlackEdgeColorButton);
 	connect(BlackEdgeColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
 
 	ResetBlackEdgeColorButton = new QToolButton(this);
 	ResetBlackEdgeColorButton->setText(tr("Reset"));
+	ResetBlackEdgeColorButton->setEnabled(mBlackEdgeColorEnabled);
 	connect(ResetBlackEdgeColorButton, SIGNAL(clicked()), this, SLOT(ResetColorButtonClicked()));
 
-	HighContrastColorLayout->addWidget(BlackEdgeColorLabel,2,0);
+	HighContrastColorLayout->addWidget(BlackEdgeColorEnabledBox,2,0);
 	HighContrastColorLayout->addWidget(BlackEdgeColorButton,2,1);
 	HighContrastColorLayout->addWidget(ResetBlackEdgeColorButton,2,2);
 
-	QLabel* DarkEdgeColorLabel = new QLabel(tr("Dark Parts Edge Color:"), this);
+	DarkEdgeColorEnabledBox = new QCheckBox(tr("Dark Parts Edge Color:"), this);
+	DarkEdgeColorEnabledBox->setChecked(mDarkEdgeColorEnabled);
+	connect(DarkEdgeColorEnabledBox, SIGNAL(clicked()), this, SLOT(ColorCheckBoxClicked()));
+
 	DarkEdgeColorButton = new QToolButton(this);
+	DarkEdgeColorButton->setEnabled(mDarkEdgeColorEnabled);
 	SetButtonPixmap(mDarkEdgeColor, DarkEdgeColorButton);
 	connect(DarkEdgeColorButton, SIGNAL(clicked()), this, SLOT(ColorButtonClicked()));
 
 	ResetDarkEdgeColorButton = new QToolButton(this);
 	ResetDarkEdgeColorButton->setText(tr("Reset"));
+	ResetDarkEdgeColorButton->setEnabled(mDarkEdgeColorEnabled);
 	connect(ResetDarkEdgeColorButton, SIGNAL(clicked()), this, SLOT(ResetColorButtonClicked()));
 
-	HighContrastColorLayout->addWidget(DarkEdgeColorLabel,3,0);
+	HighContrastColorLayout->addWidget(DarkEdgeColorEnabledBox,3,0);
 	HighContrastColorLayout->addWidget(DarkEdgeColorButton,3,1);
 	HighContrastColorLayout->addWidget(ResetDarkEdgeColorButton,3,2);
 
@@ -140,6 +164,35 @@ lcAutomateEdgeColorDialog::lcAutomateEdgeColorDialog(QWidget* Parent, bool ShowH
 	setMinimumSize(220,100);
 }
 
+void lcAutomateEdgeColorDialog::ColorCheckBoxClicked()
+{
+	QObject* CheckBox = sender();
+	if (CheckBox == StudCylinderColorEnabledBox)
+	{
+		mStudCylinderColorEnabled = StudCylinderColorEnabledBox->isChecked();
+		StudCylinderColorButton->setEnabled(mStudCylinderColorEnabled);
+		ResetStudCylinderColorButton->setEnabled(mStudCylinderColorEnabled);
+	}
+	else if (CheckBox == PartEdgeColorEnabledBox)
+	{
+		mPartEdgeColorEnabled = PartEdgeColorEnabledBox->isChecked();
+		PartEdgeColorButton->setEnabled(mPartEdgeColorEnabled);
+		ResetPartEdgeColorButton->setEnabled(mPartEdgeColorEnabled);
+	}
+	else if (CheckBox == BlackEdgeColorEnabledBox)
+	{
+		mBlackEdgeColorEnabled = BlackEdgeColorEnabledBox->isChecked();
+		BlackEdgeColorButton->setEnabled(mBlackEdgeColorEnabled);
+		ResetBlackEdgeColorButton->setEnabled(mBlackEdgeColorEnabled);
+	}
+	else if (CheckBox == DarkEdgeColorEnabledBox)
+	{
+		mDarkEdgeColorEnabled = DarkEdgeColorEnabledBox->isChecked();
+		DarkEdgeColorButton->setEnabled(mDarkEdgeColorEnabled);
+		ResetDarkEdgeColorButton->setEnabled(mDarkEdgeColorEnabled);
+	}
+}
+
 void lcAutomateEdgeColorDialog::SliderValueChanged(int Value)
 {
 	if (sender() == PartEdgeContrastSlider)
