@@ -5,7 +5,7 @@
 #include "lc_math.h"
 
 #define LC_FOURCC(ch0, ch1, ch2, ch3) (quint32)((quint32)(quint8)(ch0) | ((quint32)(quint8)(ch1) << 8) | \
-												((quint32)(quint8)(ch2) << 16) | ((quint32)(quint8)(ch3) << 24 ))
+												((quint32)(quint8)(ch2) << 16) | ((quint32)(quint8)(ch3) << 24))
 
 class lcFile
 {
@@ -156,7 +156,7 @@ public:
 	lcVector3 ReadVector3()
 	{
 		lcVector3 Vector;
-		ReadFloats(Vector, 3);
+		ReadFloats(Vector.GetFloats(), 3);
 		return Vector;
 	}
 
@@ -277,7 +277,7 @@ public:
 
 	void WriteVector3(const lcVector3& Vector)
 	{
-		WriteFloats(Vector, 3);
+		WriteFloats(Vector.GetFloats(), 3);
 	}
 
 	void WriteQString(const QString& String)
