@@ -2,7 +2,6 @@
 
 #include "lc_application.h"
 #include "lc_shortcuts.h"
-#include "lc_array.h"
 #include "lc_commands.h"
 #include "lc_model.h"
 
@@ -10,13 +9,11 @@ class lcPartSelectionWidget;
 class lcPreviewDockWidget;
 class PiecePreview;
 class lcQPartsTree;
-class lcQColorList;
-class lcQPropertiesTree;
+class lcColorList;
+class lcPropertiesWidget;
 class lcTimelineWidget;
 class lcElidedLabel;
-#ifdef QT_NO_PRINTER
-class QPrinter;
-#endif
+enum class lcRenderDialogMode;
 
 #define LC_MAX_RECENT_FILES 4
 
@@ -231,7 +228,6 @@ public:
 	void SetAngleSnapIndex(int Index);
 	void SetRelativeTransform(bool RelativeTransform);
 	void SetSeparateTransform(bool SelectionTransform);
-	void SetCurrentPieceInfo(PieceInfo* Info);
 	void SetShadingMode(lcShadingMode ShadingMode);
 	void SetSelectionMode(lcSelectionMode SelectionMode);
 	void ToggleViewSphere();
@@ -271,11 +267,10 @@ public:
 	void UpdateSnap();
 	void UpdateColor();
 	void UpdateUndoRedo(const QString& UndoText, const QString& RedoText);
-	void UpdatePerspective();
-	void UpdateCameraMenu();
 	void UpdateShadingMode();
 	void UpdateSelectionMode();
 	void UpdateModels();
+	void UpdateInUseCategory();
 	void UpdateCategories();
 	void UpdateTitle();
 	void UpdateModified(bool Modified);
@@ -292,10 +287,12 @@ public slots:
 	void ProjectFileChanged(const QString& Path);
 	void PreviewPiece(const QString& PartId, int ColorCode, bool ShowPreview);
 	void TogglePreviewWidget(bool Visible);
+	void SetCurrentPieceInfo(PieceInfo* Info);
 
 protected slots:
+	void CameraMenuAboutToShow();
+	void ProjectionMenuAboutToShow();
 	void ViewFocusReceived();
-	void ViewCameraChanged();
 	void UpdateDockWidgetActions();
 	void UpdateGamepads();
 	void ModelTabContextMenuRequested(const QPoint& Point);
@@ -305,6 +302,7 @@ protected slots:
 	void ClipboardChanged();
 	void ActionTriggered();
 	void ColorChanged(int ColorIndex);
+	void PartListPicked(PieceInfo* Info);
 	void ColorButtonClicked();
 	void Print(QPrinter* Printer);
 	void EnableWindowFlags(bool);
@@ -326,7 +324,8 @@ protected:
 	void ShowUpdatesDialog();
 	void ShowAboutDialog();
 	void ShowHTMLDialog();
-	void ShowRenderDialog();
+	void ShowExportPOVRayDialog();
+	void ShowRenderDialog(lcRenderDialogMode RenderDialogMode);
 	void ShowInstructionsDialog();
 	void ShowPrintDialog();
 	void CreatePreviewWidget();
@@ -376,9 +375,9 @@ protected:
 	QDockWidget* mTimelineToolBar;
 
 	lcPartSelectionWidget* mPartSelectionWidget;
-	lcQColorList* mColorList;
+	lcColorList* mColorList;
 	QToolButton* mColorButton;
-	lcQPropertiesTree* mPropertiesWidget;
+	lcPropertiesWidget* mPropertiesWidget;
 	lcTimelineWidget* mTimelineWidget;
 	QLineEdit* mTransformXEdit;
 	QLineEdit* mTransformYEdit;
