@@ -51,6 +51,31 @@ inline T lcClamp(const T& Value, const T& Min, const T& Max)
 		return Value;
 }
 
+class lcVector2i
+{
+public:
+	lcVector2i()
+	{
+	}
+
+	constexpr lcVector2i(const int _x, const int _y)
+		: x(_x), y(_y)
+	{
+	}
+
+	int x, y;
+};
+
+inline bool operator==(const lcVector2i& a, const lcVector2i& b)
+{
+	return a.x == b.x && a.y == b.y;
+}
+
+inline bool operator!=(const lcVector2i& a, const lcVector2i& b)
+{
+	return a.x != b.x || a.y != b.y;
+}
+
 class lcVector2
 {
 public:
@@ -63,16 +88,6 @@ public:
 	{
 	}
 
-	operator const float*() const
-	{
-		return (const float*)this;
-	}
-
-	operator float*()
-	{
-		return (float*)this;
-	}
-
 	const float& operator[](int i) const
 	{
 		return ((float*)this)[i];
@@ -83,6 +98,16 @@ public:
 		return ((float*)this)[i];
 	}
 
+	const float* GetFloats() const
+	{
+		return (const float*)this;
+	}
+
+	float* GetFloats()
+	{
+		return (float*)this;
+	}
+
 	bool IsNan() const
 	{
 		return std::isnan(x) || std::isnan(y);
@@ -105,16 +130,6 @@ public:
 
 	explicit lcVector3(const lcVector4& v);
 
-	operator const float*() const
-	{
-		return (const float*)this;
-	}
-
-	operator float*()
-	{
-		return (float*)this;
-	}
-
 	const float& operator[](int i) const
 	{
 		return ((float*)this)[i];
@@ -125,6 +140,16 @@ public:
 		return ((float*)this)[i];
 	}
 
+	const float* GetFloats() const
+	{
+		return (const float*)this;
+	}
+
+	float* GetFloats()
+	{
+		return (float*)this;
+	}
+
 	bool IsNan() const
 	{
 		return std::isnan(x) || std::isnan(y) || std::isnan(z);
@@ -154,16 +179,6 @@ public:
 	{
 	}
 
-	operator const float*() const
-	{
-		return (const float*)this;
-	}
-
-	operator float*()
-	{
-		return (float*)this;
-	}
-
 	const float& operator[](int i) const
 	{
 		return ((float*)this)[i];
@@ -174,6 +189,16 @@ public:
 		return ((float*)this)[i];
 	}
 
+	const float* GetFloats() const
+	{
+		return (const float*)this;
+	}
+
+	float* GetFloats()
+	{
+		return (float*)this;
+	}
+
 	bool IsNan() const
 	{
 		return std::isnan(x) || std::isnan(y) || std::isnan(z) || std::isnan(w);
@@ -198,16 +223,6 @@ public:
 
 	explicit lcMatrix33(const lcMatrix44& Matrix);
 
-	operator const float*() const
-	{
-		return (const float*)this;
-	}
-
-	operator float*()
-	{
-		return (float*)this;
-	}
-
 	const lcVector3& operator[](int i) const
 	{
 		return r[i];
@@ -218,6 +233,16 @@ public:
 		return r[i];
 	}
 
+	const float* GetFloats() const
+	{
+		return (const float*)this;
+	}
+
+	float* GetFloats()
+	{
+		return (float*)this;
+	}
+
 	void Orthonormalize();
 
 	lcVector3 r[3];
@@ -256,16 +281,6 @@ public:
 		r[3] = lcVector4(Translation[0], Translation[1], Translation[2], 1.0f);
 	}
 
-	operator const float*() const
-	{
-		return (const float*)this;
-	}
-
-	operator float*()
-	{
-		return (float*)this;
-	}
-
 	const lcVector4& operator[](int i) const
 	{
 		return r[i];
@@ -276,11 +291,26 @@ public:
 		return r[i];
 	}
 
+	const float* GetFloats() const
+	{
+		return (const float*)this;
+	}
+
+	float* GetFloats()
+	{
+		return (float*)this;
+	}
+
 	float Determinant() const;
 
 	lcVector4 r[4];
 };
 
+inline bool operator==(const lcVector2& a, const lcVector2& b)
+{
+	return a.x == b.x && a.y == b.y;
+}
+
 inline lcVector3::lcVector3(const lcVector4& v)
 	: x(v.x), y(v.y), z(v.z)
 {
@@ -404,6 +434,36 @@ inline bool operator!=(const lcVector3& a, const lcVector3& b)
 	return a.x != b.x || a.y != b.y || a.z != b.z;
 }
 
+inline bool operator<(const lcVector3& a, const lcVector3& b)
+{
+	if (a.x < b.x)
+		return true;
+	else if (a.x > b.x)
+		return false;
+
+	if (a.y < b.y)
+		return true;
+	else if (a.y > b.y)
+		return false;
+
+	if (a.z < b.z)
+		return true;
+	else if (a.z > b.z)
+		return false;
+
+	return false;
+}
+
+inline bool operator==(const lcVector4& a, const lcVector4& b)
+{
+	return a.x == b.x && a.y == b.y && a.z == b.z && a.w == b.w;
+}
+
+inline bool operator==(const lcMatrix33& a, const lcMatrix33& b)
+{
+	return a.r[0] == b.r[0] && a.r[1] == b.r[1] && a.r[2] == b.r[2];
+}
+
 #ifndef QT_NO_DEBUG
 
 inline QDebug operator<<(QDebug Debug, const lcVector2& v)
@@ -649,6 +709,11 @@ inline lcVector3 lcVector3FromColor(quint32 Color)
 	return v;
 }
 
+inline lcVector3 lcVector3FromQColor(QColor Color)
+{
+	return lcVector3(Color.redF(), Color.greenF(), Color.blueF());
+}
+
 inline lcVector4 lcVector4FromColor(quint32 Color)
 {
 	lcVector4 v(LC_RGBA_RED(Color), LC_RGBA_GREEN(Color), LC_RGBA_BLUE(Color), LC_RGBA_ALPHA(Color));
@@ -661,6 +726,11 @@ inline quint32 lcColorFromVector3(const lcVector3& Color)
 	return LC_RGB(roundf(Color[0] * 255), roundf(Color[1] * 255), roundf(Color[2] * 255));
 }
 
+inline QColor lcQColorFromVector3(const lcVector3& Color)
+{
+	return QColor::fromRgb(roundf(Color[0] * 255), roundf(Color[1] * 255), roundf(Color[2] * 255));
+}
+
 inline float lcLuminescence(const lcVector3& Color)
 {
 	return 0.2126f * Color[0] + 0.7152f * Color[1] + 0.0722f * Color[2];
@@ -935,6 +1005,19 @@ inline lcVector3 lcMatrix33ToEulerAngles(const lcMatrix33& RotMat)
 	return Rot;
 }
 
+inline bool lcMatrix33Similar(const lcMatrix33& a, const lcMatrix33& b)
+{
+	for (int Row = 0; Row < 3; Row++)
+	{
+		float Dot = lcDot(a.r[Row], b.r[Row]);
+
+		if (Dot < 0.99f || Dot > 1.01f)
+			return false;
+	}
+
+	return true;
+}
+
 inline float lcMatrix44::Determinant() const
 {
 	return r[0][0] * r[1][1] * r[2][2] + r[0][1] * r[1][2] * r[2][0] +
@@ -1506,7 +1589,7 @@ inline lcVector3 lcQuaternionMul(const lcVector3& a, const lcVector4& b)
 }
 
 // Convert world coordinates to screen coordinates.
-inline lcVector3 lcProjectPoint(const lcVector3& Point, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int Viewport[4])
+inline lcVector3 lcProjectPoint(const lcVector3& Point, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int (&Viewport)[4])
 {
 	lcVector4 Tmp;
 
@@ -1520,7 +1603,7 @@ inline lcVector3 lcProjectPoint(const lcVector3& Point, const lcMatrix44& ModelV
 	return lcVector3(Viewport[0] + (1 + Tmp[0]) * Viewport[2] / 2, Viewport[1] + (1 + Tmp[1]) * Viewport[3] / 2, (1 + Tmp[2]) / 2);
 }
 
-inline lcVector3 lcUnprojectPoint(const lcVector3& Point, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int Viewport[4])
+inline lcVector3 lcUnprojectPoint(const lcVector3& Point, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int (&Viewport)[4])
 {
 	// Calculate the screen to model transform.
 	const lcMatrix44 Transform = lcMatrix44Inverse(lcMul(ModelView, Projection));
@@ -1541,7 +1624,7 @@ inline lcVector3 lcUnprojectPoint(const lcVector3& Point, const lcMatrix44& Mode
 	return lcVector3(Tmp[0], Tmp[1], Tmp[2]);
 }
 
-inline void lcUnprojectPoints(lcVector3* Points, int NumPoints, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int Viewport[4])
+inline void lcUnprojectPoints(lcVector3* Points, int NumPoints, const lcMatrix44& ModelView, const lcMatrix44& Projection, const int (&Viewport)[4])
 {
 	// Calculate the screen to model transform.
 	const lcMatrix44 Transform = lcMatrix44Inverse(lcMul(ModelView, Projection));
@@ -1565,7 +1648,7 @@ inline void lcUnprojectPoints(lcVector3* Points, int NumPoints, const lcMatrix44
 	}
 }
 
-inline void lcGetFrustumPlanes(const lcMatrix44& WorldView, const lcMatrix44& Projection, lcVector4 Planes[6])
+inline void lcGetFrustumPlanes(const lcMatrix44& WorldView, const lcMatrix44& Projection, lcVector4 (&Planes)[6])
 {
 	lcMatrix44 WorldProj = lcMul(WorldView, Projection);
 
@@ -1795,10 +1878,10 @@ inline void lcPolygonPlaneClip(lcVector3* InPoints, int NumInPoints, lcVector3*
 }
 
 // Return true if a polygon intersects a set of planes.
-inline bool lcTriangleIntersectsPlanes(const float* p1, const float* p2, const float* p3, const lcVector4 Planes[6])
+inline bool lcTriangleIntersectsPlanes(const lcVector3& p1, const lcVector3& p2, const lcVector3& p3, const lcVector4 (&Planes)[6])
 {
 	constexpr int NumPlanes = 6;
-	const float* const Points[3] = { p1, p2, p3 };
+	const lcVector3 Points[3] = { p1, p2, p3 };
 	int Outcodes[3] = { 0, 0, 0 }, i;
 	constexpr int NumPoints = 3;
 
@@ -1991,6 +2074,125 @@ inline bool lcSphereRayMinIntersectDistance(const lcVector3& Center, float Radiu
 	}
 }
 
+inline bool lcConeRayMinIntersectDistance(const lcVector3& Tip, const lcVector3& Direction, float Radius, float Height, const lcVector3& Start, const lcVector3& End, float* Dist)
+{
+	const lcVector3 v = End - Start;
+	const lcVector3 h = Direction;
+	const lcVector3 w = Start - Tip;
+	const float vh = lcDot(v, h);
+	const float wh = lcDot(w, h);
+	const float m = (Radius * Radius) / (Height * Height);
+
+	const float a = lcDot(v, v) - m * vh * vh - vh * vh;
+	const float b = 2 * (lcDot(v, w) - m * vh * wh - vh * wh);
+	const float c = lcDot(w, w) - m * wh * wh - wh * wh;
+
+	const float delta = b * b - 4 * a * c;
+
+	if (delta < 0.0f)
+		return false;
+
+	float ts[2] = { (-b - sqrtf(delta)) / (2 * a), (-b + sqrtf(delta)) / (2 * a) };
+
+	for (int ti = 0; ti < 2; ti++)
+	{
+		float t = ts[ti];
+		lcVector3 Intersection = Start + v * t;
+
+		float ConeD = lcDot(Intersection - Tip, Direction);
+
+		if (ConeD < 0.0f)
+			continue;
+
+		if (ConeD < Height)
+		{
+			*Dist = lcLength(Intersection - Start);
+
+			return true;
+		}
+
+		lcVector3 Center(Tip + Direction * Height);
+		lcVector4 Plane(Direction, -lcDot(Direction, Center));
+
+		if (lcLineSegmentPlaneIntersection(&Intersection, Start, End, Plane))
+		{
+			if (lcLengthSquared(Center - Intersection) < Radius * Radius)
+			{
+				*Dist = lcLength(Intersection - Start);
+
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
+
+inline bool lcCylinderRayMinIntersectDistance(float Radius, float Height, const lcVector3& Start, const lcVector3& End, float* Dist)
+{
+	lcVector4 BottomPlane(0.0f, 0.0f, 1.0f, 0.0f);
+	lcVector3 Intersection;
+	float MinDistance = FLT_MAX;
+
+	if (lcLineSegmentPlaneIntersection(&Intersection, Start, End, BottomPlane))
+	{
+		if (Intersection.x * Intersection.x + Intersection.y * Intersection.y < Radius * Radius)
+		{
+			float Distance = lcLength(Intersection - Start);
+
+			if (Distance < MinDistance)
+				MinDistance = Distance;
+		}
+	}
+
+	lcVector4 TopPlane(0.0f, 0.0f, 1.0f, -Height);
+
+	if (lcLineSegmentPlaneIntersection(&Intersection, Start, End, TopPlane))
+	{
+		if (Intersection.x * Intersection.x + Intersection.y * Intersection.y < Radius * Radius)
+		{
+			float Distance = lcLength(Intersection - Start);
+
+			if (Distance < MinDistance)
+				MinDistance = Distance;
+		}
+	}
+
+	lcVector3 Direction = End - Start;
+
+	float a = (Direction.x * Direction.x) + (Direction.y * Direction.y);
+	float b = 2 * (Direction.x * Start.x + Direction.y * Start.y);
+	float c = (Start.x * Start.x) + (Start.y * Start.y) - (Radius * Radius);
+
+	float delta = b * b - 4 * (a * c);
+
+	if (delta > 0.0f)
+	{
+		float ts[2] = { (-b - sqrtf(delta)) / (2 * a), (-b + sqrtf(delta)) / (2 * a) };
+
+		for (int ti = 0; ti < 2; ti++)
+		{
+			float t = ts[ti];
+			Intersection = Start + Direction * t;
+
+			if (Intersection.z < 0.0f || Intersection.z > Height)
+				continue;
+
+			float Distance = lcLength(Intersection - Start);
+
+			if (Distance < MinDistance)
+				MinDistance = Distance;
+		}
+	}
+
+	if (MinDistance == FLT_MAX)
+		return false;
+
+	*Dist = MinDistance;
+
+	return true;
+}
+
 inline lcVector3 lcRayPointClosestPoint(const lcVector3& Point, const lcVector3& Start, const lcVector3& End)
 {
 	const lcVector3 Dir = Point - Start;
@@ -2010,7 +2212,7 @@ inline float lcRayPointDistance(const lcVector3& Point, const lcVector3& Start,
 }
 
 // Returns true if the axis aligned box intersects the volume defined by planes.
-inline bool lcBoundingBoxIntersectsVolume(const lcVector3& Min, const lcVector3& Max, const lcVector4 Planes[6])
+inline bool lcBoundingBoxIntersectsVolume(const lcVector3& Min, const lcVector3& Max, const lcVector4 (&Planes)[6])
 {
 	constexpr int NumPlanes = 6;
 	lcVector3 Points[8] =
@@ -2085,7 +2287,7 @@ struct lcBoundingBox
 	lcVector3 Max;
 };
 
-inline void lcGetBoxCorners(const lcVector3& Min, const lcVector3& Max, lcVector3 Points[8])
+inline void lcGetBoxCorners(const lcVector3& Min, const lcVector3& Max, lcVector3 (&Points)[8])
 {
 	Points[0] = lcVector3(Max.x, Max.y, Min.z);
 	Points[1] = lcVector3(Min.x, Max.y, Min.z);
@@ -2097,7 +2299,7 @@ inline void lcGetBoxCorners(const lcVector3& Min, const lcVector3& Max, lcVector
 	Points[7] = lcVector3(Min.x, Max.y, Max.z);
 }
 
-inline void lcGetBoxCorners(const lcBoundingBox& BoundingBox, lcVector3 Points[8])
+inline void lcGetBoxCorners(const lcBoundingBox& BoundingBox, lcVector3 (&Points)[8])
 {
 	lcGetBoxCorners(BoundingBox.Min, BoundingBox.Max, Points);
 }
@@ -2237,3 +2439,5 @@ inline lcVector4 lcAlgorithmicEdgeColor(const lcVector3& Value, const float Valu
 
 	return  lcVector4(lcLinearToSRGB(rgbf), 1.0f);
 }
+
+Q_DECLARE_METATYPE(lcVector3);
