@@ -75,6 +75,10 @@ enum LC_PROFILE_KEY
 	LC_PROFILE_PARTS_LIST_DECORATED,
 	LC_PROFILE_PARTS_LIST_ALIASES,
 	LC_PROFILE_PARTS_LIST_LISTMODE,
+	LC_PROFILE_PARTS_LIST_PART_FILTER,
+	LC_PROFILE_PARTS_LIST_CASE_SENSITIVE_FILTER,
+	LC_PROFILE_PARTS_LIST_FILE_NAME_FILTER,
+	LC_PROFILE_PARTS_LIST_PART_DESCRIPTION_FILTER,
 	LC_PROFILE_STUD_STYLE,
 
 	// Defaults for new projects.
@@ -88,17 +92,28 @@ enum LC_PROFILE_KEY
 	LC_PROFILE_HTML_IMAGE_HEIGHT,
 	LC_PROFILE_POVRAY_PATH,
 	LC_PROFILE_POVRAY_LGEO_PATH,
-	LC_PROFILE_POVRAY_WIDTH,
-	LC_PROFILE_POVRAY_HEIGHT,
+	LC_PROFILE_RENDER_WIDTH,
+	LC_PROFILE_RENDER_HEIGHT,
+	LC_PROFILE_RENDER_QUALITY,
+	LC_PROFILE_BLENDER_PATH,
+	LC_PROFILE_BLENDER_LDRAW_CONFIG_PATH,
+	LC_PROFILE_BLENDER_VERSION,
+	LC_PROFILE_BLENDER_ADDON_VERSION,
+	LC_PROFILE_BLENDER_ADDON_VERSION_CHECK,
+	LC_PROFILE_BLENDER_IMPORT_MODULE,
 
 	LC_PROFILE_PREVIEW_VIEW_SPHERE_ENABLED,
 	LC_PROFILE_PREVIEW_VIEW_SPHERE_SIZE,
 	LC_PROFILE_PREVIEW_VIEW_SPHERE_LOCATION,
 	LC_PROFILE_PREVIEW_DRAW_AXES,
 
+	LC_PROFILE_STUD_CYLINDER_COLOR_ENABLED,
 	LC_PROFILE_STUD_CYLINDER_COLOR,
+	LC_PROFILE_PART_EDGE_COLOR_ENABLED,
 	LC_PROFILE_PART_EDGE_COLOR,
+	LC_PROFILE_BLACK_EDGE_COLOR_ENABLED,
 	LC_PROFILE_BLACK_EDGE_COLOR,
+	LC_PROFILE_DARK_EDGE_COLOR_ENABLED,
 	LC_PROFILE_DARK_EDGE_COLOR,
 	LC_PROFILE_PART_EDGE_CONTRAST,
 	LC_PROFILE_PART_COLOR_VALUE_LD_INDEX,
