@@ -1,7 +1,6 @@
 #pragma once
 
 #include "lc_mesh.h"
-#include "lc_array.h"
 
 enum class lcRenderMeshState : int
 {
@@ -90,7 +89,7 @@ public:
 
 	void AddInterfaceObject(const lcObject* Object)
 	{
-		mInterfaceObjects.Add(Object);
+		mInterfaceObjects.emplace_back(Object);
 	}
 
 	void Draw(lcContext* Context) const;
@@ -115,8 +114,8 @@ protected:
 	bool mTranslucentFade;
 
 	std::function<void()> mPreTranslucentCallback;
-	lcArray<lcRenderMesh> mRenderMeshes;
-	lcArray<int> mOpaqueMeshes;
-	lcArray<lcTranslucentMeshInstance> mTranslucentMeshes;
-	lcArray<const lcObject*> mInterfaceObjects;
+	std::vector<lcRenderMesh> mRenderMeshes;
+	std::vector<int> mOpaqueMeshes;
+	std::vector<lcTranslucentMeshInstance> mTranslucentMeshes;
+	std::vector<const lcObject*> mInterfaceObjects;
 };
