@@ -40,6 +40,9 @@ public:
 	};
 
 	lcToolShortcut mShortcuts[static_cast<int>(lcTool::Count)];
+
+private:
+	void Clear();
 };
 
 extern lcMouseShortcuts gMouseShortcuts;
