@@ -128,12 +128,12 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 		StepItem->setExpanded(true);
 	}
 
-	const lcArray<lcPiece*>& Pieces = Model->GetPieces();
+	const std::vector<std::unique_ptr<lcPiece>>& Pieces = Model->GetPieces();
 	QTreeWidgetItem* StepItem = nullptr;
 	int PieceItemIndex = 0;
 	lcStep Step = 0;
 
-	for (lcPiece* Piece : Pieces)
+	for (const std::unique_ptr<lcPiece>& Piece : Pieces)
 	{
 		while (Step != Piece->GetStepShow())
 		{
@@ -143,8 +143,9 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 				{
 					QTreeWidgetItem* PieceItem = StepItem->child(PieceItemIndex);
 					lcPiece* RemovePiece = (lcPiece*)PieceItem->data(0, Qt::UserRole).value<uintptr_t>();
+					auto RemoveIt = std::find_if(Pieces.begin(), Pieces.end(), [RemovePiece](const std::unique_ptr<lcPiece>& CheckPiece){ return CheckPiece.get() == RemovePiece; });
 
-					if (Pieces.FindIndex(RemovePiece) == -1)
+					if (RemoveIt == Pieces.end())
 					{
 						mItems.remove(RemovePiece);
 						delete PieceItem;
@@ -162,7 +163,7 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 			PieceItemIndex = 0;
 		}
 
-		QTreeWidgetItem* PieceItem = mItems.value(Piece);
+		QTreeWidgetItem* PieceItem = mItems.value(Piece.get());
 		bool UpdateItem = UpdateItems;
 
 		if (StepItem)
@@ -171,9 +172,9 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 			{
 				PieceItem = new QTreeWidgetItem();
 				PieceItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
-				PieceItem->setData(0, Qt::UserRole, QVariant::fromValue<uintptr_t>((uintptr_t)Piece));
+				PieceItem->setData(0, Qt::UserRole, QVariant::fromValue<uintptr_t>((uintptr_t)Piece.get()));
 				StepItem->insertChild(PieceItemIndex, PieceItem);
-				mItems[Piece] = PieceItem;
+				mItems[Piece.get()] = PieceItem;
 
 				UpdateItem = true;
 			}
@@ -202,7 +203,7 @@ void lcTimelineWidget::Update(bool Clear, bool UpdateItems)
 
 				QImage Image(Size, Size, QImage::Format_ARGB32);
 				Image.fill(0);
-				float* Color = gColorList[ColorIndex].Value;
+				const lcVector4& Color = gColorList[ColorIndex].Value;
 				QPainter Painter(&Image);
 				Painter.setPen(Qt::darkGray);
 				Painter.setBrush(QColor::fromRgbF(Color[0], Color[1], Color[2]));
@@ -523,7 +524,7 @@ void lcTimelineWidget::CurrentItemChanged(QTreeWidgetItem* Current, QTreeWidgetI
 
 void lcTimelineWidget::ItemSelectionChanged()
 {
-	lcArray<lcObject*> Selection;
+	std::vector<lcObject*> Selection;
 	lcStep LastStep = 1;
 	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
 
@@ -533,23 +534,26 @@ void lcTimelineWidget::ItemSelectionChanged()
 		if (Piece)
 		{
 			LastStep = lcMax(LastStep, Piece->GetStepShow());
-			Selection.Add(Piece);
+			Selection.emplace_back(Piece);
 		}
 	}
 
 	lcPiece* CurrentPiece = nullptr;
 	QTreeWidgetItem* CurrentItem = currentItem();
+
 	if (CurrentItem && CurrentItem->isSelected())
 		CurrentPiece = (lcPiece*)CurrentItem->data(0, Qt::UserRole).value<uintptr_t>();
 
 	bool Blocked = blockSignals(true);
 	mIgnoreUpdates = true;
 	lcModel* Model = gMainWindow->GetActiveModel();
+
 	if (LastStep > Model->GetCurrentStep())
 	{
 		Model->SetCurrentStep(LastStep);
 		UpdateCurrentStepItem();
 	}
+
 	Model->SetSelectionAndFocus(Selection, CurrentPiece, LC_PIECE_SECTION_POSITION, false);
 	mIgnoreUpdates = false;
 	blockSignals(Blocked);
@@ -558,19 +562,19 @@ void lcTimelineWidget::ItemSelectionChanged()
 void lcTimelineWidget::dropEvent(QDropEvent* Event)
 {
 	QTreeWidgetItem* DropItem = itemAt(Event->pos());
-	lcModel* Model = gMainWindow->GetActiveModel();
+
+	if (!DropItem)
+		return;
 
 	QList<QTreeWidgetItem*> SelectedItems = selectedItems();
 	clearSelection();
 
-	if (DropItem)
-	{
-		QTreeWidgetItem* ParentItem = DropItem->parent();
-		lcStep Step = indexOfTopLevelItem(ParentItem ? ParentItem : DropItem) + 1;
+	QTreeWidgetItem* ParentItem = DropItem->parent();
+	lcStep Step = indexOfTopLevelItem(ParentItem ? ParentItem : DropItem) + 1;
+	lcModel* Model = gMainWindow->GetActiveModel();
 
-		if (Step > Model->GetCurrentStep())
-			Model->SetCurrentStep(Step);
-	}
+	if (Step > Model->GetCurrentStep())
+		Model->SetCurrentStep(Step);
 
 	auto SortItems = [this](QTreeWidgetItem* Item1, QTreeWidgetItem* Item2)
 	{
@@ -641,7 +645,7 @@ void lcTimelineWidget::PreviewSelection(QTreeWidgetItem* CurrentItem)
 
 void lcTimelineWidget::UpdateModel()
 {
-	QList<QPair<lcPiece*, lcStep>> PieceSteps;
+	std::vector<std::pair<lcPiece*, lcStep>> PieceSteps;
 
 	for (int TopLevelItemIdx = 0; TopLevelItemIdx < topLevelItemCount(); TopLevelItemIdx++)
 	{
@@ -652,7 +656,7 @@ void lcTimelineWidget::UpdateModel()
 			QTreeWidgetItem* PieceItem = StepItem->child(PieceItemIdx);
 			lcPiece* Piece = (lcPiece*)PieceItem->data(0, Qt::UserRole).value<uintptr_t>();
 
-			PieceSteps.append(QPair<lcPiece*, lcStep>(Piece, TopLevelItemIdx + 1));
+			PieceSteps.emplace_back(std::pair<lcPiece*, lcStep>(Piece, TopLevelItemIdx + 1));
 		}
 	}
 
