@@ -3,20 +3,25 @@
 #include "lc_viewwidget.h"
 #include <stdlib.h>
 #include "lc_mainwindow.h"
+#include "light.h"
 #include "camera.h"
 #include "texfont.h"
 #include "lc_texture.h"
 #include "piece.h"
 #include "pieceinf.h"
 #include "lc_synth.h"
+#include "lc_traintrack.h"
 #include "lc_scene.h"
 #include "lc_context.h"
 #include "lc_viewmanipulator.h"
 #include "lc_viewsphere.h"
 #include "lc_findreplacewidget.h"
+#include "lc_library.h"
+#include "lc_qutils.h"
+#include "lc_partselectionpopup.h"
 
 lcFindReplaceParams lcView::mFindReplaceParams;
-lcFindReplaceWidget* lcView::mFindWidget;
+QPointer<lcFindReplaceWidget> lcView::mFindWidget;
 
 lcView* lcView::mLastFocusedView;
 std::vector<lcView*> lcView::mViews;
@@ -47,6 +52,16 @@ lcView::lcView(lcViewType ViewType, lcModel* Model)
 
 lcView::~lcView()
 {
+	if (!mPreviewInsertPieceInfo.empty())
+	{
+		lcPiecesLibrary* Library = lcGetPiecesLibrary();
+
+		for (lcInsertPieceInfo& PreviewPieceInfoTransform : mPreviewInsertPieceInfo)
+			Library->ReleasePieceInfo(PreviewPieceInfoTransform.Info);
+
+		mPreviewInsertPieceInfo.clear();
+	}
+
 	mContext->DestroyVertexBuffer(mGridBuffer);
 
 	if (gMainWindow && mViewType == lcViewType::View)
@@ -211,7 +226,6 @@ void lcView::RemoveCamera()
 	else
 		mCamera->SetViewpoint(lcViewpoint::Home);
 
-	emit CameraChanged();
 	Redraw();
 }
 
@@ -295,6 +309,64 @@ lcMatrix44 lcView::GetTileProjectionMatrix(int CurrentRow, int CurrentColumn, in
 		return lcMatrix44Frustum(Left, Right, Bottom, Top, Near, Far);
 }
 
+void lcView::ShowTrainTrackPopup()
+{
+	if (mViewType != lcViewType::View)
+		return;
+
+	lcModel* ActiveModel = GetActiveModel();
+	lcObject* Focus = ActiveModel->GetFocusObject();
+
+	if (!Focus || !Focus->IsPiece())
+		return;
+
+	lcPiece* Piece = (lcPiece*)Focus;
+	const lcTrainTrackInfo* TrainTrackInfo = Piece->mPieceInfo->GetTrainTrackInfo();
+
+	if (!TrainTrackInfo)
+		return;
+
+	if (mTrackToolSection < LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST)
+		return;
+
+	int ConnectionIndex = mTrackToolSection - LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST;
+	const lcTrainTrackConnectionType& ConnectionType = TrainTrackInfo->GetConnections()[ConnectionIndex].Type;
+	std::vector<PieceInfo*> TrainTrackParts = lcGetPiecesLibrary()->GetVisibleTrainTrackParts(ConnectionType);
+
+	auto PartSortFunc = [](const PieceInfo* a, const PieceInfo* b)
+	{
+		if (!a)
+			return true;
+
+		if (!b)
+			return false;
+
+		return strcmp(a->m_strDescription, b->m_strDescription) < 0;
+	};
+
+	std::sort(TrainTrackParts.begin(), TrainTrackParts.end(), PartSortFunc);
+
+	std::vector<std::pair<PieceInfo*, std::string>> Parts;
+
+	Parts.reserve(TrainTrackParts.size());
+
+	for (PieceInfo* Info : TrainTrackParts)
+		Parts.emplace_back(Info, std::string());
+
+	std::optional<PieceInfo*> Result = lcShowPartSelectionPopup(nullptr, Parts, gMainWindow->mColorIndex, mWidget, QCursor::pos());
+	PieceInfo* Info = Result.has_value() ? Result.value() : nullptr;
+
+	if (Info)
+	{
+		Piece = ActiveModel->AddPiece(Info, mTrackToolSection);
+
+		if (Piece)
+			ActiveModel->UpdateTrainTrackConnections(Piece, false);
+	}
+
+	mWidget->UpdateMousePosition();
+}
+
 void lcView::ShowContextMenu() const
 {
 	if (mViewType != lcViewType::View)
@@ -410,15 +482,60 @@ lcVector3 lcView::GetMoveDirection(const lcVector3& Direction) const
 	return axis;
 }
 
-lcMatrix44 lcView::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info) const
+void lcView::UpdatePiecePreview()
+{
+	PieceInfo* PreviewInfo = gMainWindow->GetCurrentPieceInfo();
+	std::vector<lcInsertPieceInfo> InsertPieceInfo;
+	bool IsConnection;
+
+	if (PreviewInfo)
+	{
+		std::tie(InsertPieceInfo, IsConnection) = GetMouseInsertPieceInfo(false, true, PreviewInfo, nullptr);
+
+		if (GetActiveModel() != mModel)
+		{
+			for (lcInsertPieceInfo& InfoTransform : InsertPieceInfo)
+				InfoTransform.Transform = lcMul(InfoTransform.Transform, mActiveSubmodelTransform);
+		}
+	}
+
+	lcPiecesLibrary* Library = lcGetPiecesLibrary();
+
+	for (lcInsertPieceInfo& InfoTransform : InsertPieceInfo)
+		Library->LoadPieceInfo(InfoTransform.Info, true, true);
+
+	for (lcInsertPieceInfo& PreviewPieceInfoTransform : mPreviewInsertPieceInfo)
+		Library->ReleasePieceInfo(PreviewPieceInfoTransform.Info);
+
+	mPreviewInsertPieceInfo = std::move(InsertPieceInfo);
+}
+
+std::pair<std::vector<lcInsertPieceInfo>, bool> lcView::GetMouseInsertPieceInfo(bool IgnoreSelected, bool AllowNewPieces, PieceInfo* Info, lcPiece* MovingPiece) const
 {
 	lcModel* ActiveModel = GetActiveModel();
 
-	lcPieceInfoRayTest PieceInfoRayTest = FindPieceInfoUnderPointer(IgnoreSelected);
-		
-	if (PieceInfoRayTest.Info)
+	lcObjectRayTest ObjectRayTest = RayTest(true, IgnoreSelected);
+
+	if (ObjectRayTest.ObjectSection.Object)
 	{
-		lcVector3 Position = PieceInfoRayTest.Plane;
+		lcPiece* MousePiece = reinterpret_cast<lcPiece*>(ObjectRayTest.ObjectSection.Object);
+		lcTrainTrackInfo* RayTrackInfo = MousePiece->mPieceInfo->GetTrainTrackInfo();
+		lcTrainTrackInfo* InfoTrackInfo = Info->GetTrainTrackInfo();
+
+		if (RayTrackInfo && InfoTrackInfo)
+		{
+			ActiveModel->UpdateTrainTrackConnections(MousePiece, IgnoreSelected);
+
+			lcVector3 ClosestPoint = ObjectRayTest.Start + lcNormalize(ObjectRayTest.End - ObjectRayTest.Start) * ObjectRayTest.Distance;
+			quint32 FocusSection = MousePiece->GetFocusSection();
+
+			std::vector<lcInsertPieceInfo> TrainTracks = lcTrainTrackInfo::GetInsertPieceInfo(MousePiece, Info, MovingPiece, gMainWindow->mColorIndex, FocusSection, AllowNewPieces, ClosestPoint);
+
+			if (!TrainTracks.empty())
+				return { TrainTracks, true };
+		}
+
+		lcVector3 Position = ObjectRayTest.PieceInfoRayTest.Plane;
 
 		if (Position.x > 0.0f)
 			Position.x += fabsf(Info->GetBoundingBox().Min.x);
@@ -434,14 +551,14 @@ lcMatrix44 lcView::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info)
 			Position.z -= fabsf(Info->GetBoundingBox().Max.z);
 
 		if (gMainWindow->GetRelativeTransform())
-			Position = lcMul31(ActiveModel->SnapPosition(Position), PieceInfoRayTest.Transform);
+			Position = lcMul31(ActiveModel->SnapPosition(Position), ObjectRayTest.PieceInfoRayTest.Transform);
 		else
-			Position = ActiveModel->SnapPosition(lcMul31(Position, PieceInfoRayTest.Transform));
+			Position = ActiveModel->SnapPosition(lcMul31(Position, ObjectRayTest.PieceInfoRayTest.Transform));
 
-		lcMatrix44 WorldMatrix = PieceInfoRayTest.Transform;
+		lcMatrix44 WorldMatrix = ObjectRayTest.PieceInfoRayTest.Transform;
 		WorldMatrix.SetTranslation(Position);
 
-		return WorldMatrix;
+		return { { lcInsertPieceInfo{ Info, WorldMatrix, gMainWindow->mColorIndex } }, false };
 	}
 
 	std::array<lcVector3, 2> ClickPoints = {{ lcVector3((float)mMouseX, (float)mMouseY, 0.0f), lcVector3((float)mMouseX, (float)mMouseY, 1.0f) }};
@@ -461,7 +578,7 @@ lcMatrix44 lcView::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info)
 	if (lcLineSegmentPlaneIntersection(&Intersection, ClickPoints[0], ClickPoints[1], lcVector4(0, 0, 1, BoundingBox.Min.z)))
 	{
 		Intersection = ActiveModel->SnapPosition(Intersection);
-		return lcMatrix44Translation(Intersection);
+		return { { lcInsertPieceInfo{ Info, lcMatrix44Translation(Intersection), gMainWindow->mColorIndex } }, false };
 	}
 
 	lcVector3 Position;
@@ -474,10 +591,10 @@ lcMatrix44 lcView::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info)
 	if (lcLineSegmentPlaneIntersection(&Intersection, ClickPoints[0], ClickPoints[1], lcVector4(FrontVector, -lcDot(FrontVector, Position))))
 	{
 		Intersection = ActiveModel->SnapPosition(Intersection);
-		return lcMatrix44Translation(Intersection);
+		return { { lcInsertPieceInfo{ Info, lcMatrix44Translation(Intersection), gMainWindow->mColorIndex } }, false };
 	}
 
-	return lcMatrix44Translation(UnprojectPoint(lcVector3((float)mMouseX, (float)mMouseY, 0.9f)));
+	return { { lcInsertPieceInfo{ Info, lcMatrix44Translation(UnprojectPoint(lcVector3((float)mMouseX, (float)mMouseY, 0.9f))), gMainWindow->mColorIndex } }, false };
 }
 
 lcVector3 lcView::GetCameraLightInsertPosition() const
@@ -520,7 +637,7 @@ void lcView::GetRayUnderPointer(lcVector3& Start, lcVector3& End) const
 	End = StartEnd[1];
 }
 
-lcObjectSection lcView::FindObjectUnderPointer(bool PiecesOnly, bool IgnoreSelected) const
+lcObjectRayTest lcView::RayTest(bool PiecesOnly, bool IgnoreSelected) const
 {
 	lcVector3 StartEnd[2] =
 	{
@@ -550,43 +667,15 @@ lcObjectSection lcView::FindObjectUnderPointer(bool PiecesOnly, bool IgnoreSelec
 
 	ActiveModel->RayTest(ObjectRayTest);
 
-	return ObjectRayTest.ObjectSection;
+	return ObjectRayTest;
 }
 
-lcPieceInfoRayTest lcView::FindPieceInfoUnderPointer(bool IgnoreSelected) const
+lcObjectSection lcView::FindObjectUnderPointer(bool PiecesOnly, bool IgnoreSelected) const
 {
-	lcVector3 StartEnd[2] =
-	{
-		lcVector3((float)mMouseX, (float)mMouseY, 0.0f),
-		lcVector3((float)mMouseX, (float)mMouseY, 1.0f)
-	};
-
-	UnprojectPoints(StartEnd, 2);
-
-	lcObjectRayTest ObjectRayTest;
-
-	ObjectRayTest.PiecesOnly = true;
-	ObjectRayTest.IgnoreSelected = IgnoreSelected;
-	ObjectRayTest.ViewCamera = mCamera;
-	ObjectRayTest.Start = StartEnd[0];
-	ObjectRayTest.End = StartEnd[1];
-
-	lcModel* ActiveModel = GetActiveModel();
-
-	if (ActiveModel != mModel)
-	{
-		lcMatrix44 InverseMatrix = lcMatrix44AffineInverse(mActiveSubmodelTransform);
-
-		ObjectRayTest.Start = lcMul31(ObjectRayTest.Start, InverseMatrix);
-		ObjectRayTest.End = lcMul31(ObjectRayTest.End, InverseMatrix);
-	}
-
-	ActiveModel->RayTest(ObjectRayTest);
-
-	return ObjectRayTest.PieceInfoRayTest;
+	return RayTest(PiecesOnly, IgnoreSelected).ObjectSection;
 }
 
-lcArray<lcObject*> lcView::FindObjectsInBox(float x1, float y1, float x2, float y2) const
+std::vector<lcObject*> lcView::FindObjectsInBox(float x1, float y1, float x2, float y2) const
 {
 	float Left, Top, Bottom, Right;
 
@@ -812,17 +901,10 @@ void lcView::OnDraw()
 
 	if (DrawInterface && mTrackTool == lcTrackTool::Insert)
 	{
-		PieceInfo* Info = gMainWindow->GetCurrentPieceInfo();
+		UpdatePiecePreview();
 
-		if (Info)
-		{
-			lcMatrix44 WorldMatrix = GetPieceInsertPosition(false, Info);
-
-			if (GetActiveModel() != mModel)
-				WorldMatrix = lcMul(WorldMatrix, mActiveSubmodelTransform);
-
-			Info->AddRenderMeshes(mScene.get(), WorldMatrix, gMainWindow->mColorIndex, lcRenderMeshState::Focused, false);
-		}
+		for (const lcInsertPieceInfo& PreviewPieceInfoTransform : mPreviewInsertPieceInfo)
+			PreviewPieceInfoTransform.Info->AddRenderMeshes(mScene.get(), PreviewPieceInfoTransform.Transform, PreviewPieceInfoTransform.ColorIndex, lcRenderMeshState::Focused, false);
 	}
 
 	if (DrawInterface)
@@ -918,16 +1000,16 @@ void lcView::OnDraw()
 	if (DrawOverlays)
 		DrawAxes();
 
-	if (DrawInterface)
+	if (DrawInterface && mTrackTool != lcTrackTool::Insert)
 	{
 		lcTool Tool = gMainWindow->GetTool();
 		lcModel* ActiveModel = GetActiveModel();
 
 		if ((Tool == lcTool::Select || Tool == lcTool::Move) && mTrackButton == lcTrackButton::None && ActiveModel->AnyObjectsSelected())
-			mViewManipulator->DrawSelectMove(mTrackButton, mTrackTool);
+			mViewManipulator->DrawSelectMove(mTrackButton, mTrackTool, mTrackToolSection);
 		else if (GetCurrentTool() == lcTool::Move && mTrackButton != lcTrackButton::None)
-			mViewManipulator->DrawSelectMove(mTrackButton, mTrackTool);
-		else if ((Tool == lcTool::Rotate || (Tool == lcTool::Select && mTrackButton != lcTrackButton::None && mTrackTool >= lcTrackTool::RotateX && mTrackTool <= lcTrackTool::RotateXYZ)) && ActiveModel->AnyPiecesSelected())
+			mViewManipulator->DrawSelectMove(mTrackButton, mTrackTool, mTrackToolSection);
+		else if ((Tool == lcTool::Rotate || (Tool == lcTool::Select && mTrackButton != lcTrackButton::None && mTrackTool >= lcTrackTool::RotateX && mTrackTool <= lcTrackTool::RotateXYZ)) && ActiveModel->CanRotateSelection())
 			mViewManipulator->DrawRotate(mTrackButton, mTrackTool);
 		else if ((mTrackTool == lcTrackTool::Select || mTrackTool == lcTrackTool::ZoomRegion) && mTrackButton != lcTrackButton::None)
 			DrawSelectZoomRegionOverlay();
@@ -995,10 +1077,11 @@ void lcView::DrawBackground(int CurrentTileRow, int TotalTileRows, int CurrentTi
 	}
 
 	const lcPreferences& Preferences = lcGetPreferences();
+	float Alpha = mContext->IsOffscreen() ? 0.0f : 1.0f;
 
 	if (!Preferences.mBackgroundGradient)
 	{
-		lcVector4 BackgroundColor(lcVector3FromColor(Preferences.mBackgroundSolidColor), 0.0f);
+		lcVector4 BackgroundColor(lcVector3FromColor(Preferences.mBackgroundSolidColor), Alpha);
 		mContext->ClearColorAndDepth(BackgroundColor);
 		return;
 	}
@@ -1027,28 +1110,23 @@ void lcView::DrawBackground(int CurrentTileRow, int TotalTileRows, int CurrentTi
 	double TopRed = LC_RGBA_RED(ColorTop);
 	double TopGreen = LC_RGBA_GREEN(ColorTop);
 	double TopBlue = LC_RGBA_BLUE(ColorTop);
-	double TopAlpha = LC_RGBA_ALPHA(ColorTop);
 	double BottomRed = LC_RGBA_RED(ColorBottom);
 	double BottomGreen = LC_RGBA_GREEN(ColorBottom);
 	double BottomBlue = LC_RGBA_BLUE(ColorBottom);
-	double BottomAlpha = LC_RGBA_ALPHA(ColorBottom);
 	const double DeltaRed = BottomRed - TopRed;
 	const double DeltaGreen = BottomGreen - TopGreen;
 	const double DeltaBlue = BottomBlue - TopBlue;
-	const double DeltaAlpha = BottomAlpha - TopAlpha;
 
 	BottomRed = TopRed + DeltaRed * t2;
 	BottomGreen = TopGreen + DeltaGreen * t2;
 	BottomBlue = TopBlue + DeltaBlue * t2;
-	BottomAlpha = TopAlpha + DeltaAlpha * t2;
 
 	TopRed = TopRed + DeltaRed * t1;
 	TopGreen = TopGreen + DeltaGreen * t1;
 	TopBlue = TopBlue + DeltaBlue * t1;
-	TopAlpha = TopAlpha + DeltaAlpha * t1;
 
-	const quint32 Color1 = LC_RGBA(TopRed, TopGreen, TopBlue, TopAlpha);
-	const quint32 Color2 = LC_RGBA(BottomRed, BottomGreen, BottomBlue, BottomAlpha);
+	const quint32 Color1 = LC_RGBA(TopRed, TopGreen, TopBlue, Alpha);
+	const quint32 Color2 = LC_RGBA(BottomRed, BottomGreen, BottomBlue, Alpha);
 
 	struct lcBackgroundVertex
 	{
@@ -1361,7 +1439,7 @@ void lcView::DrawRotateViewOverlay()
 	mContext->SetVertexBufferPointer(Verts);
 	mContext->SetVertexFormatPosition(2);
 
-	GLushort Indices[64 + 32] = 
+	GLushort Indices[64 + 32] =
 	{
 		0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 13, 13, 14, 14, 15, 15, 16, 16,
 		17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 23, 24, 24, 25, 25, 26, 26, 27, 27, 28, 28, 29, 29, 30, 30, 31, 31, 0,
@@ -1393,18 +1471,14 @@ void lcView::DrawGrid()
 
 	if (mTrackTool == lcTrackTool::Insert)
 	{
-		PieceInfo* CurPiece = gMainWindow->GetCurrentPieceInfo();
-
-		if (CurPiece)
+		for (const lcInsertPieceInfo& PreviewInsertPieceInfo : mPreviewInsertPieceInfo)
 		{
 			lcVector3 Points[8];
-			lcGetBoxCorners(CurPiece->GetBoundingBox(), Points);
-
-			lcMatrix44 WorldMatrix = GetPieceInsertPosition(false, CurPiece);
+			lcGetBoxCorners(PreviewInsertPieceInfo.Info->GetBoundingBox(), Points);
 
 			for (int i = 0; i < 8; i++)
 			{
-				lcVector3 Point = lcMul31(Points[i], WorldMatrix);
+				lcVector3 Point = lcMul31(Points[i], PreviewInsertPieceInfo.Transform);
 
 				Min = lcMin(Point, Min);
 				Max = lcMax(Point, Max);
@@ -1594,21 +1668,23 @@ lcTrackTool lcView::GetOverrideTrackTool(Qt::MouseButton Button) const
 
 	constexpr lcTrackTool TrackToolFromTool[] =
 	{
-	    lcTrackTool::Insert,      // lcTool::Insert
-	    lcTrackTool::PointLight,  // lcTool::Light
-	    lcTrackTool::SpotLight,   // lcTool::SpotLight
-	    lcTrackTool::Camera,      // lcTool::Camera
-	    lcTrackTool::Select,      // lcTool::Select
-	    lcTrackTool::MoveXYZ,     // lcTool::Move
-	    lcTrackTool::RotateXYZ,   // lcTool::Rotate
-	    lcTrackTool::Eraser,      // lcTool::Eraser
-	    lcTrackTool::Paint,       // lcTool::Paint
-	    lcTrackTool::ColorPicker, // lcTool::ColorPicker
-	    lcTrackTool::Zoom,        // lcTool::Zoom
-	    lcTrackTool::Pan,         // lcTool::Pan
-	    lcTrackTool::OrbitXY,     // lcTool::RotateView
-	    lcTrackTool::Roll,        // lcTool::Roll
-	    lcTrackTool::ZoomRegion   // lcTool::ZoomRegion
+	    lcTrackTool::Insert,           // lcTool::Insert
+	    lcTrackTool::PointLight,       // lcTool::PointLight
+		lcTrackTool::SpotLight,        // lcTool::SpotLight
+		lcTrackTool::DirectionalLight, // lcTool::DirectionalLight
+		lcTrackTool::AreaLight,        // lcTool::AreaLight
+		lcTrackTool::Camera,           // lcTool::Camera
+	    lcTrackTool::Select,           // lcTool::Select
+	    lcTrackTool::MoveXYZ,          // lcTool::Move
+	    lcTrackTool::RotateXYZ,        // lcTool::Rotate
+	    lcTrackTool::Eraser,           // lcTool::Eraser
+	    lcTrackTool::Paint,            // lcTool::Paint
+	    lcTrackTool::ColorPicker,      // lcTool::ColorPicker
+	    lcTrackTool::Zoom,             // lcTool::Zoom
+	    lcTrackTool::Pan,              // lcTool::Pan
+	    lcTrackTool::OrbitXY,          // lcTool::RotateView
+	    lcTrackTool::Roll,             // lcTool::Roll
+	    lcTrackTool::ZoomRegion        // lcTool::ZoomRegion
 	};
 
 	LC_ARRAY_SIZE_CHECK(TrackToolFromTool, lcTool::Count);
@@ -1655,9 +1731,8 @@ void lcView::EndDrag(bool Accept)
 
 		case lcDragState::Piece:
 			{
-				PieceInfo* Info = gMainWindow->GetCurrentPieceInfo();
-				if (Info)
-					ActiveModel->InsertPieceToolClicked(GetPieceInsertPosition(false, Info));
+				if (!mPreviewInsertPieceInfo.empty())
+					ActiveModel->InsertPieceToolClicked(mPreviewInsertPieceInfo);
 			} break;
 
 		case lcDragState::Color:
@@ -1686,8 +1761,6 @@ void lcView::SetViewpoint(lcViewpoint Viewpoint)
 	mCamera->SetViewpoint(Viewpoint);
 	ZoomExtents();
 	Redraw();
-
-	emit CameraChanged();
 }
 
 void lcView::SetViewpoint(const lcVector3& Position)
@@ -1705,8 +1778,6 @@ void lcView::SetViewpoint(const lcVector3& Position)
 	mCamera->SetViewpoint(Position);
 	ZoomExtents();
 	Redraw();
-
-	emit CameraChanged();
 }
 
 void lcView::SetViewpoint(const lcVector3& Position, const lcVector3& Target, const lcVector3& Up)
@@ -1723,8 +1794,6 @@ void lcView::SetViewpoint(const lcVector3& Position, const lcVector3& Target, co
 
 	mCamera->SetViewpoint(Position, Target, Up);
 	Redraw();
-
-	emit CameraChanged();
 }
 
 void lcView::SetCameraAngles(float Latitude, float Longitude)
@@ -1752,8 +1821,6 @@ void lcView::SetDefaultCamera()
 	mCamera->SetViewpoint(lcViewpoint::Home);
 	ZoomExtents();
 	Redraw();
-
-	emit CameraChanged();
 }
 
 void lcView::SetCamera(lcCamera* Camera, bool ForceCopy)
@@ -1776,29 +1843,28 @@ void lcView::SetCamera(lcCamera* Camera, bool ForceCopy)
 
 void lcView::SetCamera(const QString& CameraName)
 {
-	const lcArray<lcCamera*>& Cameras = mModel->GetCameras();
+	const std::vector<std::unique_ptr<lcCamera>>& Cameras = mModel->GetCameras();
 
-	for (int CameraIdx = 0; CameraIdx < Cameras.GetSize(); CameraIdx++)
+	for (size_t CameraIndex = 0; CameraIndex < Cameras.size(); CameraIndex++)
 	{
-		if (CameraName.compare(Cameras[CameraIdx]->GetName(), Qt::CaseInsensitive) == 0)
+		if (CameraName.compare(Cameras[CameraIndex]->GetName(), Qt::CaseInsensitive) == 0)
 		{
-			SetCameraIndex(CameraIdx);
+			SetCameraIndex(CameraIndex);
 			return;
 		}
 	}
 }
 
-void lcView::SetCameraIndex(int Index)
+void lcView::SetCameraIndex(size_t CameraIndex)
 {
-	const lcArray<lcCamera*>& Cameras = mModel->GetCameras();
+	const std::vector<std::unique_ptr<lcCamera>>& Cameras = mModel->GetCameras();
 
-	if (Index >= Cameras.GetSize())
+	if (CameraIndex >= Cameras.size())
 		return;
 
-	lcCamera* Camera = Cameras[Index];
+	lcCamera* Camera = Cameras[CameraIndex].get();
 	SetCamera(Camera, false);
 
-	emit CameraChanged();
 	Redraw();
 }
 
@@ -1808,9 +1874,6 @@ void lcView::SetProjection(bool Ortho)
 	{
 		mCamera->SetOrtho(Ortho);
 		Redraw();
-
-		if (gMainWindow)
-			gMainWindow->UpdatePerspective();
 	}
 	else
 	{
@@ -1844,13 +1907,18 @@ void lcView::Zoom(float Amount)
 void lcView::ZoomExtents()
 {
 	lcModel* ActiveModel = GetActiveModel();
+
 	if (ActiveModel)
-		ActiveModel->ZoomExtents(mCamera, (float)mWidth / (float)mHeight);
+	{
+		const lcMatrix44 WorldMatrix = mActiveSubmodelInstance ? mActiveSubmodelTransform : lcMatrix44Identity();
+
+		ActiveModel->ZoomExtents(mCamera, (float)mWidth / (float)mHeight, WorldMatrix);
+	}
 }
 
 lcCursor lcView::GetCursor() const
 {
-	if (mTrackButton != lcTrackButton::None)
+	if (mTrackButton != lcTrackButton::None && mTrackTool != lcTrackTool::Paint)
 		return lcCursor::Hidden;
 
 	if (mTrackTool == lcTrackTool::Select)
@@ -1864,36 +1932,42 @@ lcCursor lcView::GetCursor() const
 
 	constexpr lcCursor CursorFromTrackTool[] =
 	{
-		lcCursor::Select,      // lcTrackTool::None
-		lcCursor::Brick,       // lcTrackTool::Insert
-		lcCursor::Light,       // lcTrackTool::PointLight
-		lcCursor::Spotlight,   // lcTrackTool::SpotLight
-		lcCursor::Camera,      // lcTrackTool::Camera
-		lcCursor::Select,      // lcTrackTool::Select
-		lcCursor::Move,        // lcTrackTool::MoveX
-		lcCursor::Move,        // lcTrackTool::MoveY
-		lcCursor::Move,        // lcTrackTool::MoveZ
-		lcCursor::Move,        // lcTrackTool::MoveXY
-		lcCursor::Move,        // lcTrackTool::MoveXZ
-		lcCursor::Move,        // lcTrackTool::MoveYZ
-		lcCursor::Move,        // lcTrackTool::MoveXYZ
-		lcCursor::Rotate,      // lcTrackTool::RotateX
-		lcCursor::Rotate,      // lcTrackTool::RotateY
-		lcCursor::Rotate,      // lcTrackTool::RotateZ
-		lcCursor::Rotate,      // lcTrackTool::RotateXY
-		lcCursor::Rotate,      // lcTrackTool::RotateXYZ
-		lcCursor::Move,        // lcTrackTool::ScalePlus
-		lcCursor::Move,        // lcTrackTool::ScaleMinus
-		lcCursor::Delete,      // lcTrackTool::Eraser
-		lcCursor::Paint,       // lcTrackTool::Paint
-		lcCursor::ColorPicker, // lcTrackTool::ColorPicker
-		lcCursor::Zoom,        // lcTrackTool::Zoom
-		lcCursor::Pan,         // lcTrackTool::Pan
-		lcCursor::RotateX,     // lcTrackTool::OrbitX
-		lcCursor::RotateY,     // lcTrackTool::OrbitY
-		lcCursor::RotateView,  // lcTrackTool::OrbitXY
-		lcCursor::Roll,        // lcTrackTool::Roll
-		lcCursor::ZoomRegion   // lcTrackTool::ZoomRegion
+		lcCursor::Select,           // lcTrackTool::None
+		lcCursor::Brick,            // lcTrackTool::Insert
+		lcCursor::PointLight,       // lcTrackTool::PointLight
+		lcCursor::SpotLight,        // lcTrackTool::SpotLight
+		lcCursor::DirectionalLight, // lcTrackTool::DirectionalLight
+		lcCursor::AreaLight,        // lcTrackTool::AreaLight
+		lcCursor::Camera,           // lcTrackTool::Camera
+		lcCursor::Select,           // lcTrackTool::Select
+		lcCursor::Move,             // lcTrackTool::MoveX
+		lcCursor::Move,             // lcTrackTool::MoveY
+		lcCursor::Move,             // lcTrackTool::MoveZ
+		lcCursor::Move,             // lcTrackTool::MoveXY
+		lcCursor::Move,             // lcTrackTool::MoveXZ
+		lcCursor::Move,             // lcTrackTool::MoveYZ
+		lcCursor::Move,             // lcTrackTool::MoveXYZ
+		lcCursor::Rotate,           // lcTrackTool::RotateX
+		lcCursor::Rotate,           // lcTrackTool::RotateY
+		lcCursor::Rotate,           // lcTrackTool::RotateZ
+		lcCursor::Rotate,           // lcTrackTool::RotateXY
+		lcCursor::Rotate,           // lcTrackTool::RotateXYZ
+		lcCursor::Select,           // lcTrackTool::RotateTrainTrackRight
+		lcCursor::Select,           // lcTrackTool::RotateTrainTrackLeft
+		lcCursor::Select,           // lcTrackTool::InsertTrainTrack
+		lcCursor::Select,           // lcTrackTool::SelectTrainTrack
+		lcCursor::Move,             // lcTrackTool::ScalePlus
+		lcCursor::Move,             // lcTrackTool::ScaleMinus
+		lcCursor::Delete,           // lcTrackTool::Eraser
+		lcCursor::Paint,            // lcTrackTool::Paint
+		lcCursor::ColorPicker,      // lcTrackTool::ColorPicker
+		lcCursor::Zoom,             // lcTrackTool::Zoom
+		lcCursor::Pan,              // lcTrackTool::Pan
+		lcCursor::RotateX,          // lcTrackTool::OrbitX
+		lcCursor::RotateY,          // lcTrackTool::OrbitY
+		lcCursor::RotateView,       // lcTrackTool::OrbitXY
+		lcCursor::Roll,             // lcTrackTool::Roll
+		lcCursor::ZoomRegion        // lcTrackTool::ZoomRegion
 	};
 
 	LC_ARRAY_SIZE_CHECK(CursorFromTrackTool, lcTrackTool::Count);
@@ -1920,8 +1994,10 @@ void lcView::SetCursor(lcCursor CursorType)
 		{  0,  0, "" },                                 // lcCursor::Hidden
 		{  0,  0, "" },                                 // lcCursor::Default
 		{  8,  3, ":/resources/cursor_insert" },        // lcCursor::Brick
-		{ 15, 15, ":/resources/cursor_light" },         // lcCursor::Light
-		{  7, 10, ":/resources/cursor_spotlight" },     // lcCursor::Spotlight
+		{ 15, 15, ":/resources/cursor_light" },         // lcCursor::PointLight
+		{  7, 10, ":/resources/cursor_spotlight" },     // lcCursor::SpotLight
+		{ 15, 15, ":/resources/cursor_sunlight" },      // lcCursor::DirectionalLight
+		{ 15, 15, ":/resources/cursor_arealight" },     // lcCursor::AreaLight
 		{ 15,  9, ":/resources/cursor_camera" },        // lcCursor::Camera
 		{  0,  2, ":/resources/cursor_select" },        // lcCursor::Select
 		{  0,  2, ":/resources/cursor_select_add" },    // lcCursor::SelectAdd
@@ -1969,36 +2045,42 @@ lcTool lcView::GetCurrentTool() const
 {
 	constexpr lcTool ToolFromTrackTool[] =
 	{
-		lcTool::Select,      // lcTrackTool::None
-		lcTool::Insert,      // lcTrackTool::Insert
-		lcTool::Light,       // lcTrackTool::PointLight
-		lcTool::SpotLight,   // lcTrackTool::SpotLight
-		lcTool::Camera,      // lcTrackTool::Camera
-		lcTool::Select,      // lcTrackTool::Select
-		lcTool::Move,        // lcTrackTool::MoveX
-		lcTool::Move,        // lcTrackTool::MoveY
-		lcTool::Move,        // lcTrackTool::MoveZ
-		lcTool::Move,        // lcTrackTool::MoveXY
-		lcTool::Move,        // lcTrackTool::MoveXZ
-		lcTool::Move,        // lcTrackTool::MoveYZ
-		lcTool::Move,        // lcTrackTool::MoveXYZ
-		lcTool::Rotate,      // lcTrackTool::RotateX
-		lcTool::Rotate,      // lcTrackTool::RotateY
-		lcTool::Rotate,      // lcTrackTool::RotateZ
-		lcTool::Rotate,      // lcTrackTool::RotateXY
-		lcTool::Rotate,      // lcTrackTool::RotateXYZ
-		lcTool::Move,        // lcTrackTool::ScalePlus
-		lcTool::Move,        // lcTrackTool::ScaleMinus
-		lcTool::Eraser,      // lcTrackTool::Eraser
-		lcTool::Paint,       // lcTrackTool::Paint
-		lcTool::ColorPicker, // lcTrackTool::ColorPicker
-		lcTool::Zoom,        // lcTrackTool::Zoom
-		lcTool::Pan,         // lcTrackTool::Pan
-		lcTool::RotateView,  // lcTrackTool::OrbitX
-		lcTool::RotateView,  // lcTrackTool::OrbitY
-		lcTool::RotateView,  // lcTrackTool::OrbitXY
-		lcTool::Roll,        // lcTrackTool::Roll
-		lcTool::ZoomRegion   // lcTrackTool::ZoomRegion
+		lcTool::Select,           // lcTrackTool::None
+		lcTool::Insert,           // lcTrackTool::Insert
+		lcTool::PointLight,       // lcTrackTool::PointLight
+		lcTool::SpotLight,        // lcTrackTool::SpotLight
+		lcTool::DirectionalLight, // lcTrackTool::DirectionalLight
+		lcTool::AreaLight,        // lcTrackTool::AreaLight
+		lcTool::Camera,           // lcTrackTool::Camera
+		lcTool::Select,           // lcTrackTool::Select
+		lcTool::Move,             // lcTrackTool::MoveX
+		lcTool::Move,             // lcTrackTool::MoveY
+		lcTool::Move,             // lcTrackTool::MoveZ
+		lcTool::Move,             // lcTrackTool::MoveXY
+		lcTool::Move,             // lcTrackTool::MoveXZ
+		lcTool::Move,             // lcTrackTool::MoveYZ
+		lcTool::Move,             // lcTrackTool::MoveXYZ
+		lcTool::Rotate,           // lcTrackTool::RotateX
+		lcTool::Rotate,           // lcTrackTool::RotateY
+		lcTool::Rotate,           // lcTrackTool::RotateZ
+		lcTool::Rotate,           // lcTrackTool::RotateXY
+		lcTool::Rotate,           // lcTrackTool::RotateXYZ
+		lcTool::Rotate,           // lcTrackTool::RotateTrainTrackRight
+		lcTool::Rotate,           // lcTrackTool::RotateTrainTrackLeft
+		lcTool::Insert,           // lcTrackTool::InsertTrainTrack
+		lcTool::Select,           // lcTrackTool::SelectTrainTrack
+		lcTool::Move,             // lcTrackTool::ScalePlus
+		lcTool::Move,             // lcTrackTool::ScaleMinus
+		lcTool::Eraser,           // lcTrackTool::Eraser
+		lcTool::Paint,            // lcTrackTool::Paint
+		lcTool::ColorPicker,      // lcTrackTool::ColorPicker
+		lcTool::Zoom,             // lcTrackTool::Zoom
+		lcTool::Pan,              // lcTrackTool::Pan
+		lcTool::RotateView,       // lcTrackTool::OrbitX
+		lcTool::RotateView,       // lcTrackTool::OrbitY
+		lcTool::RotateView,       // lcTrackTool::OrbitXY
+		lcTool::Roll,             // lcTrackTool::Roll
+		lcTool::ZoomRegion        // lcTrackTool::ZoomRegion
 	};
 
 	LC_ARRAY_SIZE_CHECK(ToolFromTrackTool, lcTrackTool::Count);
@@ -2020,9 +2102,11 @@ void lcView::UpdateTrackTool()
 
 	lcTool CurrentTool = gMainWindow->GetTool();
 	lcTrackTool NewTrackTool = mTrackTool;
+	quint32 NewTrackSection = ~0U;
 	int x = mMouseX;
 	int y = mMouseY;
 	bool Redraw = false;
+	mMouseDownPiece = nullptr;
 	mTrackToolFromOverlay = false;
 	lcModel* ActiveModel = GetActiveModel();
 
@@ -2032,7 +2116,7 @@ void lcView::UpdateTrackTool()
 		NewTrackTool = lcTrackTool::Insert;
 		break;
 
-	case lcTool::Light:
+	case lcTool::PointLight:
 		NewTrackTool = lcTrackTool::PointLight;
 		break;
 
@@ -2040,6 +2124,14 @@ void lcView::UpdateTrackTool()
 		NewTrackTool = lcTrackTool::SpotLight;
 		break;
 
+	case lcTool::DirectionalLight:
+		NewTrackTool = lcTrackTool::DirectionalLight;
+		break;
+
+	case lcTool::AreaLight:
+		NewTrackTool = lcTrackTool::AreaLight;
+		break;
+
 	case lcTool::Camera:
 		NewTrackTool = lcTrackTool::Camera;
 		break;
@@ -2047,10 +2139,9 @@ void lcView::UpdateTrackTool()
 	case lcTool::Select:
 	case lcTool::Move:
 		{
-			mMouseDownPiece = nullptr;
-			NewTrackTool = mViewManipulator->UpdateSelectMove();
+			std::tie(NewTrackTool, NewTrackSection) = mViewManipulator->UpdateSelectMove(mTrackButton);
 			mTrackToolFromOverlay = NewTrackTool != lcTrackTool::MoveXYZ && NewTrackTool != lcTrackTool::Select;
-			Redraw = NewTrackTool != mTrackTool;
+			Redraw = NewTrackTool != mTrackTool || NewTrackSection != mTrackToolSection;
 
 			if (CurrentTool == lcTool::Select && NewTrackTool == lcTrackTool::Select && mMouseModifiers == Qt::NoModifier)
 			{
@@ -2060,8 +2151,8 @@ void lcView::UpdateTrackTool()
 				if (Object && Object->IsPiece() && ObjectSection.Section == LC_PIECE_SECTION_POSITION && Object->IsSelected())
 				{
 					lcPiece* Piece = (lcPiece*)Object;
-					mMouseDownPosition = Piece->mModelWorld.GetTranslation();
-					mMouseDownPiece = Piece->mPieceInfo;
+					mMouseDownTransform = Piece->mModelWorld;
+					mMouseDownPiece = Piece;
 					NewTrackTool = lcTrackTool::MoveXYZ;
 				}
 			}
@@ -2165,6 +2256,8 @@ void lcView::UpdateTrackTool()
 	}
 
 	mTrackTool = NewTrackTool;
+	mTrackToolSection = NewTrackSection;
+
 	UpdateCursor();
 
 	if (Redraw)
@@ -2258,16 +2351,11 @@ void lcView::StartTracking(lcTrackButton TrackButton)
 	switch (Tool)
 	{
 		case lcTool::Insert:
-		case lcTool::Light:
-			break;
-
+		case lcTool::PointLight:
 		case lcTool::SpotLight:
-		{
-			lcVector3 Position = GetCameraLightInsertPosition();
-			lcVector3 Target = Position + lcVector3(0.1f, 0.1f, 0.1f);
-			ActiveModel->BeginSpotLightTool(Position, Target);
-		}
-		break;
+		case lcTool::DirectionalLight:
+		case lcTool::AreaLight:
+			break;
 
 		case lcTool::Camera:
 		{
@@ -2322,10 +2410,12 @@ void lcView::StopTracking(bool Accept)
 	switch (Tool)
 	{
 	case lcTool::Insert:
-	case lcTool::Light:
+	case lcTool::PointLight:
 		break;
 
 	case lcTool::SpotLight:
+	case lcTool::DirectionalLight:
+	case lcTool::AreaLight:
 	case lcTool::Camera:
 		ActiveModel->EndMouseTool(Tool, Accept);
 		break;
@@ -2333,7 +2423,7 @@ void lcView::StopTracking(bool Accept)
 	case lcTool::Select:
 		if (Accept && mMouseDownX != mMouseX && mMouseDownY != mMouseY)
 		{
-			lcArray<lcObject*> Objects = FindObjectsInBox(mMouseDownX, mMouseDownY, mMouseX, mMouseY);
+			std::vector<lcObject*> Objects = FindObjectsInBox(mMouseDownX, mMouseDownY, mMouseX, mMouseY);
 
 			if (mMouseModifiers & Qt::ControlModifier)
 				ActiveModel->AddToSelection(Objects, true, true);
@@ -2418,6 +2508,17 @@ void lcView::OnButtonDown(lcTrackButton TrackButton)
 	lcModel* ActiveModel = GetActiveModel();
 	mToolClicked = false;
 
+	auto AddLight = [this, ActiveModel](lcLightType LightType)
+	{
+		ActiveModel->InsertLightToolClicked(GetCameraLightInsertPosition(), LightType);
+
+		if ((mMouseModifiers & Qt::ControlModifier) == 0)
+			gMainWindow->SetTool(lcTool::Select);
+
+		mToolClicked = true;
+		UpdateTrackTool();
+	};
+
 	switch (mTrackTool)
 	{
 	case lcTrackTool::None:
@@ -2425,12 +2526,16 @@ void lcView::OnButtonDown(lcTrackButton TrackButton)
 
 	case lcTrackTool::Insert:
 		{
-			PieceInfo* CurPiece = gMainWindow->GetCurrentPieceInfo();
+			UpdatePiecePreview();
 
-			if (!CurPiece)
+			if (mPreviewInsertPieceInfo.empty())
 				break;
 
-			ActiveModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
+			if (GetActiveModel() != mModel)
+				for (lcInsertPieceInfo& PreviewPieceInfoTransform : mPreviewInsertPieceInfo)
+					PreviewPieceInfoTransform.Transform = lcMul(PreviewPieceInfoTransform.Transform, lcMatrix44AffineInverse(mActiveSubmodelTransform));
+
+			ActiveModel->InsertPieceToolClicked(mPreviewInsertPieceInfo);
 
 			if ((mMouseModifiers & Qt::ControlModifier) == 0)
 				gMainWindow->SetTool(lcTool::Select);
@@ -2441,22 +2546,25 @@ void lcView::OnButtonDown(lcTrackButton TrackButton)
 		break;
 
 	case lcTrackTool::PointLight:
-		{
-			ActiveModel->PointLightToolClicked(GetCameraLightInsertPosition());
-
-			if ((mMouseModifiers & Qt::ControlModifier) == 0)
-				gMainWindow->SetTool(lcTool::Select);
-
-			mToolClicked = true;
-			UpdateTrackTool();
-		}
+		AddLight(lcLightType::Point);
 		break;
 
 	case lcTrackTool::SpotLight:
+		AddLight(lcLightType::Spot);
+		break;
+
+	case lcTrackTool::DirectionalLight:
+		AddLight(lcLightType::Directional);
+		break;
+
+	case lcTrackTool::AreaLight:
+		AddLight(lcLightType::Area);
+		break;
+
 	case lcTrackTool::Camera:
 		StartTracking(TrackButton);
 		break;
-		
+
 	case lcTrackTool::Select:
 		{
 			lcObjectSection ObjectSection = FindObjectUnderPointer(false, false);
@@ -2488,10 +2596,54 @@ void lcView::OnButtonDown(lcTrackButton TrackButton)
 	case lcTrackTool::RotateZ:
 	case lcTrackTool::RotateXY:
 	case lcTrackTool::RotateXYZ:
-		if (ActiveModel->AnyPiecesSelected())
+		if (ActiveModel->CanRotateSelection())
 			StartTracking(TrackButton);
 		break;
 
+	case lcTrackTool::RotateTrainTrackRight:
+	case lcTrackTool::RotateTrainTrackLeft:
+		{
+			ActiveModel->RotateFocusedTrainTrack(mTrackTool == lcTrackTool::RotateTrainTrackRight ? 1 : -1);
+
+			mToolClicked = true;
+			UpdateTrackTool();
+		}
+		break;
+
+	case lcTrackTool::InsertTrainTrack:
+		{
+			ShowTrainTrackPopup();
+
+			mToolClicked = true;
+			UpdateTrackTool();
+		}
+		break;
+
+	case lcTrackTool::SelectTrainTrack:
+		{
+			lcObject* Focus = ActiveModel->GetFocusObject();
+
+			if (!Focus || !Focus->IsPiece())
+				break;
+
+			lcPiece* Piece = (lcPiece*)Focus;
+			const lcTrainTrackInfo* TrainTrackInfo = Piece->mPieceInfo->GetTrainTrackInfo();
+
+			if (!TrainTrackInfo)
+				break;
+
+			if (mTrackToolSection < LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST)
+				break;
+
+			lcObjectSection ObjectSection;
+
+			ObjectSection.Object = Focus;
+			ObjectSection.Section = mTrackToolSection;
+
+			ActiveModel->ClearSelectionAndSetFocus(ObjectSection, true);
+		}
+		break;
+
 	case lcTrackTool::ScalePlus:
 	case lcTrackTool::ScaleMinus:
 		if (ActiveModel->AnyPiecesSelected())
@@ -2505,6 +2657,7 @@ void lcView::OnButtonDown(lcTrackButton TrackButton)
 
 	case lcTrackTool::Paint:
 		ActiveModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
+		StartTracking(TrackButton);
 		mToolClicked = true;
 		break;
 
@@ -2566,9 +2719,20 @@ void lcView::OnLeftButtonDoubleClick()
 		return;
 	}
 
-	lcObjectSection ObjectSection = FindObjectUnderPointer(false, false);
 	lcModel* ActiveModel = GetActiveModel();
 
+	if (mTrackTool == lcTrackTool::RotateTrainTrackRight || mTrackTool == lcTrackTool::RotateTrainTrackLeft)
+	{
+		ActiveModel->RotateFocusedTrainTrack(mTrackTool == lcTrackTool::RotateTrainTrackRight ? 1 : -1);
+
+		mToolClicked = true;
+		UpdateTrackTool();
+
+		return;
+	}
+
+	lcObjectSection ObjectSection = FindObjectUnderPointer(false, false);
+
 	if (mMouseModifiers & Qt::ControlModifier)
 		ActiveModel->FocusOrDeselectObject(ObjectSection);
 	else if (mMouseModifiers & Qt::ShiftModifier)
@@ -2687,10 +2851,9 @@ void lcView::OnMouseMove()
 	case lcTrackTool::None:
 	case lcTrackTool::Insert:
 	case lcTrackTool::PointLight:
-		break;
-
 	case lcTrackTool::SpotLight:
-		ActiveModel->UpdateSpotLightTool(GetCameraLightInsertPosition());
+	case lcTrackTool::DirectionalLight:
+	case lcTrackTool::AreaLight:
 		break;
 
 	case lcTrackTool::Camera:
@@ -2788,9 +2951,15 @@ void lcView::OnMouseMove()
 			}
 			else if (mTrackTool == lcTrackTool::MoveXYZ && mMouseDownPiece)
 			{
-				lcMatrix44 NewPosition = GetPieceInsertPosition(true, mMouseDownPiece);
-				lcVector3 Distance = NewPosition.GetTranslation() - mMouseDownPosition;
-				ActiveModel->UpdateMoveTool(Distance, false, mTrackButton != lcTrackButton::Left);
+				if (mMouseDownPiece->mPieceInfo->GetTrainTrackInfo())
+					ActiveModel->UpdateSelectedPiecesTrainTrackConnections();
+
+				auto [PieceInfoTransforms, IsConnection] = GetMouseInsertPieceInfo(true, false, mMouseDownPiece->mPieceInfo, mMouseDownPiece);
+
+				if (!PieceInfoTransforms.empty())
+				{
+					ActiveModel->UpdateFreeMoveTool(mMouseDownPiece, mMouseDownTransform, PieceInfoTransforms.back().Transform, IsConnection, mTrackButton != lcTrackButton::Left);
+				}
 			}
 			else if (mTrackTool == lcTrackTool::ScalePlus || mTrackTool == lcTrackTool::ScaleMinus)
 			{
@@ -2925,8 +3094,17 @@ void lcView::OnMouseMove()
 		}
 		break;
 
+	case lcTrackTool::RotateTrainTrackRight:
+	case lcTrackTool::RotateTrainTrackLeft:
+	case lcTrackTool::InsertTrainTrack:
+	case lcTrackTool::SelectTrainTrack:
 	case lcTrackTool::Eraser:
+		break;
+
 	case lcTrackTool::Paint:
+		ActiveModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
+		break;
+
 	case lcTrackTool::ColorPicker:
 		break;
 
