@@ -4,6 +4,8 @@
 #include "lc_math.h"
 #include "lc_commands.h"
 
+struct lcInsertPieceInfo;
+
 enum class lcDragState
 {
 	None,
@@ -17,8 +19,10 @@ enum class lcCursor
 	Hidden = First,
 	Default,
 	Brick,
-	Light,
-	Spotlight,
+	PointLight,
+	SpotLight,
+	DirectionalLight,
+	AreaLight,
 	Camera,
 	Select,
 	SelectAdd,
@@ -52,6 +56,8 @@ enum class lcTrackTool
 	Insert,
 	PointLight,
 	SpotLight,
+	DirectionalLight,
+	AreaLight,
 	Camera,
 	Select,
 	MoveX,
@@ -66,6 +72,10 @@ enum class lcTrackTool
 	RotateZ,
 	RotateXY,
 	RotateXYZ,
+	RotateTrainTrackRight,
+	RotateTrainTrackLeft,
+	InsertTrainTrack,
+	SelectTrainTrack,
 	ScalePlus,
 	ScaleMinus,
 	Eraser,
@@ -240,7 +250,7 @@ public:
 	void SetDefaultCamera();
 	void SetCamera(lcCamera* Camera, bool ForceCopy);
 	void SetCamera(const QString& CameraName);
-	void SetCameraIndex(int Index);
+	void SetCameraIndex(size_t CameraIndex);
 
 	void SetProjection(bool Ortho);
 	void LookAt();
@@ -255,12 +265,13 @@ public:
 
 	float GetOverlayScale() const;
 	lcVector3 GetMoveDirection(const lcVector3& Direction) const;
-	lcMatrix44 GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info) const;
+	void UpdatePiecePreview();
+	std::pair<std::vector<lcInsertPieceInfo>, bool> GetMouseInsertPieceInfo(bool IgnoreSelected, bool AllowNewPieces, PieceInfo* Info, lcPiece* MovingPiece) const;
 	lcVector3 GetCameraLightInsertPosition() const;
 	void GetRayUnderPointer(lcVector3& Start, lcVector3& End) const;
+	lcObjectRayTest RayTest(bool PiecesOnly, bool IgnoreSelected) const;
 	lcObjectSection FindObjectUnderPointer(bool PiecesOnly, bool IgnoreSelected) const;
-	lcPieceInfoRayTest FindPieceInfoUnderPointer(bool IgnoreSelected) const;
-	lcArray<lcObject*> FindObjectsInBox(float x1, float y1, float x2, float y2) const;
+	std::vector<lcObject*> FindObjectsInBox(float x1, float y1, float x2, float y2) const;
 
 	lcVector3 ProjectPoint(const lcVector3& Point) const;
 	lcVector3 UnprojectPoint(const lcVector3& Point) const;
@@ -280,7 +291,6 @@ public:
 
 signals:
 	void FocusReceived();
-	void CameraChanged();
 
 protected:
 	void DrawBackground(int CurrentTileRow, int TotalTileRows, int CurrentTileHeight) const;
@@ -304,6 +314,8 @@ protected:
 	void StartPan(int x, int y);
 	void UpdatePan(int x, int y);
 
+	void ShowTrainTrackPopup();
+
 	lcViewWidget* mWidget = nullptr;
 	int mWidth = 1;
 	int mHeight = 1;
@@ -324,8 +336,9 @@ protected:
 
 	lcDragState mDragState;
 	bool mTrackToolFromOverlay;
-	lcVector3 mMouseDownPosition;
-	PieceInfo* mMouseDownPiece;
+	quint32 mTrackToolSection = ~0U;
+	lcMatrix44 mMouseDownTransform;
+	lcPiece* mMouseDownPiece = nullptr;
 	int mPanX = 0;
 	int mPanY = 0;
 
@@ -344,10 +357,12 @@ protected:
 
 	lcCamera* mCamera = nullptr;
 
+	std::vector<lcInsertPieceInfo> mPreviewInsertPieceInfo;
+
 	lcVertexBuffer mGridBuffer;
 	int mGridSettings[7];
 
-	static lcFindReplaceWidget* mFindWidget;
+	static QPointer<lcFindReplaceWidget> mFindWidget;
 	static lcFindReplaceParams mFindReplaceParams;
 
 	static lcView* mLastFocusedView;
