@@ -34,6 +34,19 @@ MinifigWizard::MinifigWizard()
 {
 	LoadSettings();
 	LoadTemplates();
+
+	lcMinifigTemplate& Template = mTemplates[GetDefaultTemplateName()];
+	const int ColorCodes[LC_MFW_NUMITEMS] = { 4, 7, 14, 7, 1, 0, 7, 4, 4, 14, 14, 7, 7, 0, 0, 7, 7 };
+	const char* const Pieces[LC_MFW_NUMITEMS] = { "3624.dat", "", "3626bp01.dat", "", "973.dat", "3815.dat", "", "3819.dat", "3818.dat", "3820.dat", "3820.dat", "", "", "3817.dat", "3816.dat", "", "" };
+
+	for (int PartIndex = 0; PartIndex < LC_MFW_NUMITEMS; PartIndex++)
+	{
+		Template.Parts[PartIndex] = Pieces[PartIndex];
+		Template.ColorCodes[PartIndex] = ColorCodes[PartIndex];
+		Template.Angles[PartIndex] = 0.0f;
+	}
+
+	memset(&mMinifig, 0, sizeof(mMinifig));
 }
 
 MinifigWizard::~MinifigWizard()
@@ -47,6 +60,11 @@ MinifigWizard::~MinifigWizard()
 	SaveTemplates();
 }
 
+QString MinifigWizard::GetDefaultTemplateName()
+{
+	return QCoreApplication::translate("MinifigWizard", "Default");
+}
+
 void MinifigWizard::LoadSettings()
 {
 	QString CustomSettingsPath = lcGetProfileString(LC_PROFILE_MINIFIG_SETTINGS);
@@ -68,30 +86,6 @@ void MinifigWizard::LoadSettings()
 		ParseSettings(MinifigFile);
 }
 
-void MinifigWizard::LoadDefault()
-{
-	LC_ARRAY_SIZE_CHECK(MinifigWizard::mSectionNames, LC_MFW_NUMITEMS);
-
-	const int ColorCodes[LC_MFW_NUMITEMS] = { 4, 7, 14, 7, 1, 0, 7, 4, 4, 14, 14, 7, 7, 0, 0, 7, 7 };
-	const char* const Pieces[LC_MFW_NUMITEMS] = { "3624.dat", "", "3626bp01.dat", "", "973.dat", "3815.dat", "", "3819.dat", "3818.dat", "3820.dat", "3820.dat", "", "", "3817.dat", "3816.dat", "", "" };
-	lcPiecesLibrary* Library = lcGetPiecesLibrary();
-
-	for (int i = 0; i < LC_MFW_NUMITEMS; i++)
-	{
-		mMinifig.Colors[i] = lcGetColorIndex(ColorCodes[i]);
-        mMinifig.Angles[i] = 0.0f;
-        mMinifig.Matrices[i] = lcMatrix44Identity();
-
-		PieceInfo* Info = Library->FindPiece(Pieces[i], nullptr, false, false);
-        mMinifig.Parts[i] = Info;
-        if (Info)
-			Library->LoadPieceInfo(Info, false, true);
-	}
-
-	Library->WaitForLoadQueue();
-	Calculate();
-}
-
 void MinifigWizard::ParseSettings(lcFile& Settings)
 {
 	for (int SectionIndex = 0; SectionIndex < LC_MFW_NUMITEMS; SectionIndex++)
@@ -117,7 +111,6 @@ void MinifigWizard::ParseSettings(lcFile& Settings)
 
 		if (!FoundSection)
 		{
-
 			lcMinifigPieceInfo MinifigInfo;
 			strncpy(MinifigInfo.Description, "None", sizeof(MinifigInfo.Description));
 			MinifigInfo.Description[sizeof(MinifigInfo.Description)-1] = 0;
@@ -192,6 +185,37 @@ void MinifigWizard::ParseSettings(lcFile& Settings)
 	}
 }
 
+void MinifigWizard::LoadTemplate(const QString& TemplateName)
+{
+	const std::map<QString, lcMinifigTemplate>::iterator& TemplateIt = mTemplates.find(TemplateName);
+
+	if (TemplateIt == mTemplates.end())
+		return;
+
+	const lcMinifigTemplate& Template = TemplateIt->second;
+	lcPiecesLibrary* Library = lcGetPiecesLibrary();
+
+	for (int PartIndex = 0; PartIndex < LC_MFW_NUMITEMS; PartIndex++)
+	{
+		mMinifig.ColorIndices[PartIndex] = lcGetColorIndex(Template.ColorCodes[PartIndex]);
+		mMinifig.Angles[PartIndex] = Template.Angles[PartIndex];
+		mMinifig.Matrices[PartIndex] = lcMatrix44Identity();
+
+		if (mMinifig.Parts[PartIndex])
+			Library->ReleasePieceInfo(mMinifig.Parts[PartIndex]);
+
+		PieceInfo* Info = Library->FindPiece(Template.Parts[PartIndex].toLatin1(), nullptr, false, false);
+
+		mMinifig.Parts[PartIndex] = Info;
+
+		if (Info)
+			Library->LoadPieceInfo(Info, false, true);
+	}
+
+	Library->WaitForLoadQueue();
+	Calculate();
+}
+
 void MinifigWizard::SaveTemplate(const QString& TemplateName, const lcMinifigTemplate& Template)
 {
 	mTemplates[TemplateName] = Template;
@@ -228,7 +252,7 @@ void MinifigWizard::AddTemplatesJson(const QByteArray& TemplateData)
 			QJsonObject PartObject = TemplateObject.value(QLatin1String(mSectionNames[PartIdx])).toObject();
 
 			Template.Parts[PartIdx] = PartObject["Id"].toString();
-			Template.Colors[PartIdx] = PartObject["Color"].toInt();
+			Template.ColorCodes[PartIdx] = PartObject["Color"].toInt();
 			Template.Angles[PartIdx] = PartObject["Angle"].toDouble();
 		}
 
@@ -250,7 +274,7 @@ QByteArray MinifigWizard::GetTemplatesJson() const
 			QJsonObject PartObject;
 
 			PartObject["Id"] = Template.Parts[PartIdx];
-			PartObject["Color"] = Template.Colors[PartIdx];
+			PartObject["Color"] = Template.ColorCodes[PartIdx];
 			PartObject["Angle"] = Template.Angles[PartIdx];
 
 			TemplateObject[QLatin1String(mSectionNames[PartIdx])] = PartObject;
@@ -466,14 +490,14 @@ int MinifigWizard::GetSelectionIndex(int Type) const
 	return 0;
 }
 
-void MinifigWizard::SetSelectionIndex(int Type, int Index)
+void MinifigWizard::SetPieceInfo(int Type, PieceInfo* Info)
 {
 	lcPiecesLibrary* Library = lcGetPiecesLibrary();
 
 	if (mMinifig.Parts[Type])
 		Library->ReleasePieceInfo(mMinifig.Parts[Type]);
 
-	mMinifig.Parts[Type] = mSettings[Type][Index].Info;
+	mMinifig.Parts[Type] = Info;
 
 	if (mMinifig.Parts[Type])
 		Library->LoadPieceInfo(mMinifig.Parts[Type], true, true);
@@ -481,9 +505,9 @@ void MinifigWizard::SetSelectionIndex(int Type, int Index)
 	Calculate();
 }
 
-void MinifigWizard::SetColor(int Type, int Color)
+void MinifigWizard::SetColorIndex(int Type, int ColorIndex)
 {
-	mMinifig.Colors[Type] = Color;
+	mMinifig.ColorIndices[Type] = ColorIndex;
 
 	Calculate();
 }
