@@ -14,44 +14,39 @@
 #include "lc_scene.h"
 #include "lc_qutils.h"
 #include "lc_synth.h"
+#include "lc_traintrack.h"
 
 constexpr float LC_PIECE_CONTROL_POINT_SIZE = 10.0f;
 
 lcPiece::lcPiece(PieceInfo* Info)
 	: lcObject(lcObjectType::Piece)
 {
-	mMesh = nullptr;
 	SetPieceInfo(Info, QString(), true);
-	mFocusedSection = LC_PIECE_SECTION_INVALID;
 	mColorIndex = gDefaultColor;
 	mColorCode = 16;
 	mStepShow = 1;
 	mStepHide = LC_STEP_MAX;
 	mGroup = nullptr;
-	mFileLine = -1;
 	mPivotMatrix = lcMatrix44Identity();
 }
 
 lcPiece::lcPiece(const lcPiece& Other)
 	: lcObject(lcObjectType::Piece)
 {
-	mMesh = nullptr;
 	SetPieceInfo(Other.mPieceInfo, Other.mID, true);
 	mHidden = Other.mHidden;
 	mSelected = Other.mSelected;
-	mFocusedSection = LC_PIECE_SECTION_INVALID;
 	mColorIndex = Other.mColorIndex;
 	mColorCode = Other.mColorCode;
 	mStepShow = Other.mStepShow;
 	mStepHide = Other.mStepHide;
 	mGroup = Other.mGroup;
-	mFileLine = -1;
 
 	mPivotMatrix = Other.mPivotMatrix;
 	mPivotPointValid = Other.mPivotPointValid;
 
-	mPositionKeys = Other.mPositionKeys;
-	mRotationKeys = Other.mRotationKeys;
+	mPosition = Other.mPosition;
+	mRotation = Other.mRotation;
 	mControlPoints = Other.mControlPoints;
 
 	UpdateMesh();
@@ -83,7 +78,7 @@ void lcPiece::SetPieceInfo(PieceInfo* Info, const QString& ID, bool Wait)
 	else
 		mID.clear();
 
-	mControlPoints.RemoveAll();
+	mControlPoints.clear();
 	delete mMesh;
 	mMesh = nullptr;
 
@@ -96,6 +91,18 @@ void lcPiece::SetPieceInfo(PieceInfo* Info, const QString& ID, bool Wait)
 	}
 }
 
+bool lcPiece::SetPieceId(PieceInfo* Info)
+{
+	if (mPieceInfo == Info)
+		return false;
+
+	lcPiecesLibrary* Library = lcGetPiecesLibrary();
+	Library->ReleasePieceInfo(mPieceInfo);
+	SetPieceInfo(Info, QString(), true);
+
+	return true;
+}
+
 void lcPiece::UpdateID()
 {
 	mID = mPieceInfo->mFileName;
@@ -113,7 +120,7 @@ void lcPiece::SaveLDraw(QTextStream& Stream) const
 
 	if (mPivotPointValid)
 	{
-		const float* PivotMatrix = mPivotMatrix;
+		const float* PivotMatrix = mPivotMatrix.GetFloats();
 		const float PivotNumbers[12] = { PivotMatrix[12], -PivotMatrix[14], PivotMatrix[13], PivotMatrix[0], -PivotMatrix[8], PivotMatrix[4], -PivotMatrix[2], PivotMatrix[10], -PivotMatrix[6], PivotMatrix[1], -PivotMatrix[9], PivotMatrix[5] };
 
 		Stream << QLatin1String("0 !LEOCAD PIECE PIVOT ");
@@ -124,15 +131,12 @@ void lcPiece::SaveLDraw(QTextStream& Stream) const
 		Stream << LineEnding;
 	}
 
-	if (mPositionKeys.GetSize() > 1)
-		mPositionKeys.SaveKeysLDraw(Stream, "PIECE POSITION_KEY ");
-
-	if (mRotationKeys.GetSize() > 1)
-		mRotationKeys.SaveKeysLDraw(Stream, "PIECE ROTATION_KEY ");
+	mPosition.Save(Stream, "PIECE", "POSITION", false);
+	mRotation.Save(Stream, "PIECE", "ROTATION", false);
 
 	Stream << "1 " << mColorCode << ' ';
 
-	const float* Matrix = mModelWorld;
+	const float* Matrix = mModelWorld.GetFloats();
 	const float Numbers[12] = { Matrix[12], -Matrix[14], Matrix[13], Matrix[0], -Matrix[8], Matrix[4], -Matrix[2], Matrix[10], -Matrix[6], Matrix[1], -Matrix[9], Matrix[5] };
 
 	for (int NumberIdx = 0; NumberIdx < 12; NumberIdx++)
@@ -164,10 +168,10 @@ bool lcPiece::ParseLDrawLine(QTextStream& Stream)
 			mPivotMatrix = PivotMatrix;
 			mPivotPointValid = true;
 		}
-		else if (Token == QLatin1String("POSITION_KEY"))
-			mPositionKeys.LoadKeysLDraw(Stream);
-		else if (Token == QLatin1String("ROTATION_KEY"))
-			mRotationKeys.LoadKeysLDraw(Stream);
+		else if (mPosition.Load(Stream, Token, "POSITION"))
+			continue;
+		else if (mRotation.Load(Stream, Token, "ROTATION"))
+			continue;
 	}
 
 	return false;
@@ -202,9 +206,9 @@ bool lcPiece::FileLoad(lcFile& file)
 			file.ReadU8(&type, 1);
 
 			if (type == 0)
-				mPositionKeys.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
+				mPosition.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
 			else if (type == 1)
-				mRotationKeys.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
+				mRotation.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
 		}
 
 		file.ReadU32(&n, 1);
@@ -234,9 +238,9 @@ bool lcPiece::FileLoad(lcFile& file)
 				file.ReadU8(&type, 1);
 
 				if (type == 0)
-					mPositionKeys.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
+					mPosition.ChangeKey(lcVector3(param[0], param[1], param[2]) * PositionScale, time, true);
 				else if (type == 1)
-					mRotationKeys.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
+					mRotation.ChangeKey(lcMatrix33FromAxisAngle(lcVector3(param[0], param[1], param[2]), param[3] * LC_DTOR), time, true);
 			}
 
 			file.ReadU32(&keys, 1);
@@ -259,13 +263,13 @@ bool lcPiece::FileLoad(lcFile& file)
 
 					if (version > 3)
 					{
-						file.ReadFloats(ModelWorld, 16);
+						file.ReadFloats(ModelWorld.GetFloats(), 16);
 					}
 					else
 					{
 						lcVector3 Translation;
 						float Rotation[3];
-						file.ReadFloats(Translation, 3);
+						file.ReadFloats(Translation.GetFloats(), 3);
 						file.ReadFloats(Rotation, 3);
 						ModelWorld = lcMatrix44Translation(Translation);
 						ModelWorld = lcMul(lcMatrix44RotationZ(Rotation[2] * LC_DTOR), lcMul(lcMatrix44RotationY(Rotation[1] * LC_DTOR), lcMul(lcMatrix44RotationX(Rotation[0] * LC_DTOR), ModelWorld)));
@@ -275,8 +279,8 @@ bool lcPiece::FileLoad(lcFile& file)
 					file.ReadU8(&b, 1);
 					time = b;
 
-					mPositionKeys.ChangeKey(ModelWorld.GetTranslation() * PositionScale, 1, true);
-					mRotationKeys.ChangeKey(lcMatrix33(ModelWorld), time, true);
+					mPosition.ChangeKey(ModelWorld.GetTranslation() * PositionScale, 1, true);
+					mRotation.ChangeKey(lcMatrix33(ModelWorld), time, true);
 
 					qint32 bl;
 					file.ReadS32(&bl, 1);
@@ -286,13 +290,13 @@ bool lcPiece::FileLoad(lcFile& file)
 			{
 				lcVector3 Translation;
 				float Rotation[3];
-				file.ReadFloats(Translation, 3);
+				file.ReadFloats(Translation.GetFloats(), 3);
 				file.ReadFloats(Rotation, 3);
 				lcMatrix44 ModelWorld = lcMatrix44Translation(Translation);
 				ModelWorld = lcMul(lcMatrix44RotationZ(Rotation[2] * LC_DTOR), lcMul(lcMatrix44RotationY(Rotation[1] * LC_DTOR), lcMul(lcMatrix44RotationX(Rotation[0] * LC_DTOR), ModelWorld)));
 
-				mPositionKeys.ChangeKey(lcVector3(ModelWorld.r[3][0], ModelWorld.r[3][1], ModelWorld.r[3][2]) * PositionScale, 1, true);
-				mRotationKeys.ChangeKey(lcMatrix33(ModelWorld), 1, true);
+				mPosition.ChangeKey(lcVector3(ModelWorld.r[3][0], ModelWorld.r[3][1], ModelWorld.r[3][2]) * PositionScale, 1, true);
+				mRotation.ChangeKey(lcMatrix33(ModelWorld), 1, true);
 			}
 		}
 	}
@@ -387,10 +391,8 @@ void lcPiece::Initialize(const lcMatrix44& WorldMatrix, lcStep Step)
 {
 	mStepShow = Step;
 
-	if (mPositionKeys.IsEmpty())
-		mPositionKeys.ChangeKey(WorldMatrix.GetTranslation(), 1, true);
-	if (mRotationKeys.IsEmpty())
-		mRotationKeys.ChangeKey(lcMatrix33(WorldMatrix), 1, true);
+	mPosition.SetValue(WorldMatrix.GetTranslation());
+	mRotation.SetValue(lcMatrix33(WorldMatrix));
 
 	UpdatePosition(Step);
 }
@@ -424,8 +426,8 @@ void lcPiece::InsertTime(lcStep Start, lcStep Time)
 		}
 	}
 
-	mPositionKeys.InsertTime(Start, Time);
-	mRotationKeys.InsertTime(Start, Time);
+	mPosition.InsertTime(Start, Time);
+	mRotation.InsertTime(Start, Time);
 }
 
 void lcPiece::RemoveTime(lcStep Start, lcStep Time)
@@ -457,8 +459,8 @@ void lcPiece::RemoveTime(lcStep Start, lcStep Time)
 		}
 	}
 
-	mPositionKeys.RemoveTime(Start, Time);
-	mRotationKeys.RemoveTime(Start, Time);
+	mPosition.RemoveTime(Start, Time);
+	mRotation.RemoveTime(Start, Time);
 }
 
 void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
@@ -486,14 +488,14 @@ void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
 		}
 	}
 
-	if (AreControlPointsVisible())
+	if (mPieceInfo->GetSynthInfo() && AreControlPointsVisible())
 	{
 		const lcVector3 Min(-LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE);
 		const lcVector3 Max(LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE);
 
-		for (int ControlPointIdx = 0; ControlPointIdx < mControlPoints.GetSize(); ControlPointIdx++)
+		for (quint32 ControlPointIndex = 0; ControlPointIndex < mControlPoints.size(); ControlPointIndex++)
 		{
-			const lcMatrix44 InverseTransform = lcMatrix44AffineInverse(mControlPoints[ControlPointIdx].Transform);
+			const lcMatrix44 InverseTransform = lcMatrix44AffineInverse(mControlPoints[ControlPointIndex].Transform);
 			const lcVector3 PointStart = lcMul31(Start, InverseTransform);
 			const lcVector3 PointEnd = lcMul31(End, InverseTransform);
 
@@ -503,7 +505,7 @@ void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
 			if (lcBoundingBoxRayIntersectDistance(Min, Max, PointStart, PointEnd, &Distance, nullptr, &Plane))
 			{
 				ObjectRayTest.ObjectSection.Object = const_cast<lcPiece*>(this);
-				ObjectRayTest.ObjectSection.Section = LC_PIECE_SECTION_CONTROL_POINT_FIRST + ControlPointIdx;
+				ObjectRayTest.ObjectSection.Section = LC_PIECE_SECTION_CONTROL_POINT_FIRST + ControlPointIndex;
 				ObjectRayTest.Distance = Distance;
 				ObjectRayTest.PieceInfoRayTest.Plane = Plane;
 			}
@@ -514,7 +516,7 @@ void lcPiece::RayTest(lcObjectRayTest& ObjectRayTest) const
 void lcPiece::BoxTest(lcObjectBoxTest& ObjectBoxTest) const
 {
 	if (mPieceInfo->BoxTest(mModelWorld, ObjectBoxTest.Planes))
-		ObjectBoxTest.Objects.Add(const_cast<lcPiece*>(this));
+		ObjectBoxTest.Objects.emplace_back(const_cast<lcPiece*>(this));
 }
 
 void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
@@ -607,63 +609,302 @@ void lcPiece::DrawInterface(lcContext* Context, const lcScene& Scene) const
 		Context->DrawIndexedPrimitives(GL_LINES, 24, GL_UNSIGNED_SHORT, 0);
 	}
 
-	if (!mControlPoints.IsEmpty() && AreControlPointsVisible())
+	if (mPieceInfo->GetSynthInfo())
+		DrawSynthInterface(Context, WorldMatrix);
+}
+
+void lcPiece::DrawSynthInterface(lcContext* Context, const lcMatrix44& WorldMatrix) const
+{
+	if (mControlPoints.empty() || !AreControlPointsVisible())
+		return;
+
+	float Verts[8 * 3];
+	float* CurVert = Verts;
+
+	lcVector3 CubeMin(-LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE);
+	lcVector3 CubeMax(LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE);
+
+	*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
+	*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
+	*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
+	*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
+	*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
+	*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
+	*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
+	*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
+
+	const GLushort Indices[36] =
 	{
-		float Verts[8 * 3];
-		float* CurVert = Verts;
+		0, 1, 2, 0, 2, 3, 7, 6, 5, 7, 5, 4, 5, 1, 0, 4, 5, 0,
+		7, 3, 2, 6, 7, 2, 0, 3, 7, 0, 7, 4, 6, 2, 1, 5, 6, 1
+	};
 
-		lcVector3 CubeMin(-LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE, -LC_PIECE_CONTROL_POINT_SIZE);
-		lcVector3 CubeMax(LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE, LC_PIECE_CONTROL_POINT_SIZE);
+	Context->EnableColorBlend(true);
+	Context->EnableCullFace(true);
 
-		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
-		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
-		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMin[2];
-		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMin[2];
-		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
-		*CurVert++ = CubeMin[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
-		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMax[1]; *CurVert++ = CubeMax[2];
-		*CurVert++ = CubeMax[0]; *CurVert++ = CubeMin[1]; *CurVert++ = CubeMax[2];
+	const lcPreferences& Preferences = lcGetPreferences();
+	const lcVector4 ControlPointColor = lcVector4FromColor(Preferences.mControlPointColor);
+	const lcVector4 ControlPointFocusedColor = lcVector4FromColor(Preferences.mControlPointFocusedColor);
 
-		const GLushort Indices[36] =
-		{
-			0, 1, 2, 0, 2, 3, 7, 6, 5, 7, 5, 4, 5, 1, 0, 4, 5, 0,
-			7, 3, 2, 6, 7, 2, 0, 3, 7, 0, 7, 4, 6, 2, 1, 5, 6, 1
-		};
+	for (quint32 ControlPointIndex = 0; ControlPointIndex < mControlPoints.size(); ControlPointIndex++)
+	{
+		Context->SetWorldMatrix(lcMul(mControlPoints[ControlPointIndex].Transform, WorldMatrix));
 
-		Context->EnableColorBlend(true);
-		Context->EnableCullFace(true);
+		Context->SetVertexBufferPointer(Verts);
+		Context->SetVertexFormatPosition(3);
+		Context->SetIndexBufferPointer(Indices);
 
-		const lcVector4 ControlPointColor = lcVector4FromColor(Preferences.mControlPointColor);
-		const lcVector4 ControlPointFocusedColor = lcVector4FromColor(Preferences.mControlPointFocusedColor);
+		if (IsFocused(LC_PIECE_SECTION_CONTROL_POINT_FIRST + ControlPointIndex))
+			Context->SetColor(ControlPointFocusedColor);
+		else
+			Context->SetColor(ControlPointColor);
 
-		for (int ControlPointIdx = 0; ControlPointIdx < mControlPoints.GetSize(); ControlPointIdx++)
-		{
-			Context->SetWorldMatrix(lcMul(mControlPoints[ControlPointIdx].Transform, WorldMatrix));
-
-			Context->SetVertexBufferPointer(Verts);
-			Context->SetVertexFormatPosition(3);
-			Context->SetIndexBufferPointer(Indices);
-
-			if (IsFocused(LC_PIECE_SECTION_CONTROL_POINT_FIRST + ControlPointIdx))
-				Context->SetColor(ControlPointFocusedColor);
-			else
-				Context->SetColor(ControlPointColor);
-
-			Context->DrawIndexedPrimitives(GL_TRIANGLES, 36, GL_UNSIGNED_SHORT, 0);
-		}
-
-		Context->EnableCullFace(false);
-		Context->EnableColorBlend(false);
+		Context->DrawIndexedPrimitives(GL_TRIANGLES, 36, GL_UNSIGNED_SHORT, 0);
 	}
+
+	Context->EnableCullFace(false);
+	Context->EnableColorBlend(false);
+}
+
+QVariant lcPiece::GetPropertyValue(lcObjectPropertyId PropertyId) const
+{
+	switch (PropertyId)
+	{
+	case lcObjectPropertyId::PieceId:
+		return QVariant::fromValue<void*>(mPieceInfo);
+
+	case lcObjectPropertyId::PieceColor:
+		return GetColorIndex();
+
+	case lcObjectPropertyId::PieceStepShow:
+	case lcObjectPropertyId::PieceStepHide:
+	case lcObjectPropertyId::CameraName:
+	case lcObjectPropertyId::CameraType:
+	case lcObjectPropertyId::CameraFOV:
+	case lcObjectPropertyId::CameraNear:
+	case lcObjectPropertyId::CameraFar:
+	case lcObjectPropertyId::CameraPositionX:
+	case lcObjectPropertyId::CameraPositionY:
+	case lcObjectPropertyId::CameraPositionZ:
+	case lcObjectPropertyId::CameraTargetX:
+	case lcObjectPropertyId::CameraTargetY:
+	case lcObjectPropertyId::CameraTargetZ:
+	case lcObjectPropertyId::CameraUpX:
+	case lcObjectPropertyId::CameraUpY:
+	case lcObjectPropertyId::CameraUpZ:
+	case lcObjectPropertyId::LightName:
+	case lcObjectPropertyId::LightType:
+	case lcObjectPropertyId::LightColor:
+	case lcObjectPropertyId::LightBlenderPower:
+	case lcObjectPropertyId::LightPOVRayPower:
+	case lcObjectPropertyId::LightCastShadow:
+	case lcObjectPropertyId::LightPOVRayFadeDistance:
+	case lcObjectPropertyId::LightPOVRayFadePower:
+	case lcObjectPropertyId::LightBlenderRadius:
+	case lcObjectPropertyId::LightBlenderAngle:
+	case lcObjectPropertyId::LightAreaSizeX:
+	case lcObjectPropertyId::LightAreaSizeY:
+	case lcObjectPropertyId::LightSpotConeAngle:
+	case lcObjectPropertyId::LightSpotPenumbraAngle:
+	case lcObjectPropertyId::LightPOVRaySpotTightness:
+	case lcObjectPropertyId::LightAreaShape:
+	case lcObjectPropertyId::LightPOVRayAreaGridX:
+	case lcObjectPropertyId::LightPOVRayAreaGridY:
+	case lcObjectPropertyId::ObjectPositionX:
+	case lcObjectPropertyId::ObjectPositionY:
+	case lcObjectPropertyId::ObjectPositionZ:
+	case lcObjectPropertyId::ObjectRotationX:
+	case lcObjectPropertyId::ObjectRotationY:
+	case lcObjectPropertyId::ObjectRotationZ:
+	case lcObjectPropertyId::Count:
+		break;
+	}
+
+	return QVariant();
+}
+
+bool lcPiece::SetPropertyValue(lcObjectPropertyId PropertyId, lcStep Step, bool AddKey, QVariant Value)
+{
+	Q_UNUSED(Step);
+	Q_UNUSED(AddKey);
+
+	switch (PropertyId)
+	{
+	case lcObjectPropertyId::PieceId:
+		return SetPieceId(static_cast<PieceInfo*>(Value.value<void*>()));
+
+	case lcObjectPropertyId::PieceColor:
+		return SetColorIndex(Value.toInt());
+
+	case lcObjectPropertyId::PieceStepShow:
+	case lcObjectPropertyId::PieceStepHide:
+	case lcObjectPropertyId::CameraName:
+	case lcObjectPropertyId::CameraType:
+	case lcObjectPropertyId::CameraFOV:
+	case lcObjectPropertyId::CameraNear:
+	case lcObjectPropertyId::CameraFar:
+	case lcObjectPropertyId::CameraPositionX:
+	case lcObjectPropertyId::CameraPositionY:
+	case lcObjectPropertyId::CameraPositionZ:
+	case lcObjectPropertyId::CameraTargetX:
+	case lcObjectPropertyId::CameraTargetY:
+	case lcObjectPropertyId::CameraTargetZ:
+	case lcObjectPropertyId::CameraUpX:
+	case lcObjectPropertyId::CameraUpY:
+	case lcObjectPropertyId::CameraUpZ:
+	case lcObjectPropertyId::LightName:
+	case lcObjectPropertyId::LightType:
+	case lcObjectPropertyId::LightColor:
+	case lcObjectPropertyId::LightBlenderPower:
+	case lcObjectPropertyId::LightPOVRayPower:
+	case lcObjectPropertyId::LightCastShadow:
+	case lcObjectPropertyId::LightPOVRayFadeDistance:
+	case lcObjectPropertyId::LightPOVRayFadePower:
+	case lcObjectPropertyId::LightBlenderRadius:
+	case lcObjectPropertyId::LightBlenderAngle:
+	case lcObjectPropertyId::LightAreaSizeX:
+	case lcObjectPropertyId::LightAreaSizeY:
+	case lcObjectPropertyId::LightSpotConeAngle:
+	case lcObjectPropertyId::LightSpotPenumbraAngle:
+	case lcObjectPropertyId::LightPOVRaySpotTightness:
+	case lcObjectPropertyId::LightAreaShape:
+	case lcObjectPropertyId::LightPOVRayAreaGridX:
+	case lcObjectPropertyId::LightPOVRayAreaGridY:
+	case lcObjectPropertyId::ObjectPositionX:
+	case lcObjectPropertyId::ObjectPositionY:
+	case lcObjectPropertyId::ObjectPositionZ:
+	case lcObjectPropertyId::ObjectRotationX:
+	case lcObjectPropertyId::ObjectRotationY:
+	case lcObjectPropertyId::ObjectRotationZ:
+	case lcObjectPropertyId::Count:
+		break;
+	}
+
+	return false;
+}
+
+bool lcPiece::HasKeyFrame(lcObjectPropertyId PropertyId, lcStep Time) const
+{
+	switch (PropertyId)
+	{
+	case lcObjectPropertyId::PieceId:
+	case lcObjectPropertyId::PieceColor:
+	case lcObjectPropertyId::PieceStepShow:
+	case lcObjectPropertyId::PieceStepHide:
+	case lcObjectPropertyId::CameraName:
+	case lcObjectPropertyId::CameraType:
+	case lcObjectPropertyId::CameraFOV:
+	case lcObjectPropertyId::CameraNear:
+	case lcObjectPropertyId::CameraFar:
+	case lcObjectPropertyId::CameraPositionX:
+	case lcObjectPropertyId::CameraPositionY:
+	case lcObjectPropertyId::CameraPositionZ:
+	case lcObjectPropertyId::CameraTargetX:
+	case lcObjectPropertyId::CameraTargetY:
+	case lcObjectPropertyId::CameraTargetZ:
+	case lcObjectPropertyId::CameraUpX:
+	case lcObjectPropertyId::CameraUpY:
+	case lcObjectPropertyId::CameraUpZ:
+	case lcObjectPropertyId::LightName:
+	case lcObjectPropertyId::LightType:
+	case lcObjectPropertyId::LightColor:
+	case lcObjectPropertyId::LightBlenderPower:
+	case lcObjectPropertyId::LightPOVRayPower:
+	case lcObjectPropertyId::LightCastShadow:
+	case lcObjectPropertyId::LightPOVRayFadeDistance:
+	case lcObjectPropertyId::LightPOVRayFadePower:
+	case lcObjectPropertyId::LightBlenderRadius:
+	case lcObjectPropertyId::LightBlenderAngle:
+	case lcObjectPropertyId::LightAreaSizeX:
+	case lcObjectPropertyId::LightAreaSizeY:
+	case lcObjectPropertyId::LightSpotConeAngle:
+	case lcObjectPropertyId::LightSpotPenumbraAngle:
+	case lcObjectPropertyId::LightPOVRaySpotTightness:
+	case lcObjectPropertyId::LightAreaShape:
+	case lcObjectPropertyId::LightPOVRayAreaGridX:
+	case lcObjectPropertyId::LightPOVRayAreaGridY:
+		return false;
+
+	case lcObjectPropertyId::ObjectPositionX:
+	case lcObjectPropertyId::ObjectPositionY:
+	case lcObjectPropertyId::ObjectPositionZ:
+		return mPosition.HasKeyFrame(Time);
+
+	case lcObjectPropertyId::ObjectRotationX:
+	case lcObjectPropertyId::ObjectRotationY:
+	case lcObjectPropertyId::ObjectRotationZ:
+		return mRotation.HasKeyFrame(Time);
+
+	case lcObjectPropertyId::Count:
+		return false;
+	}
+
+	return false;
+}
+
+bool lcPiece::SetKeyFrame(lcObjectPropertyId PropertyId, lcStep Time, bool KeyFrame)
+{
+	switch (PropertyId)
+	{
+	case lcObjectPropertyId::PieceId:
+	case lcObjectPropertyId::PieceColor:
+	case lcObjectPropertyId::PieceStepShow:
+	case lcObjectPropertyId::PieceStepHide:
+	case lcObjectPropertyId::CameraName:
+	case lcObjectPropertyId::CameraType:
+	case lcObjectPropertyId::CameraFOV:
+	case lcObjectPropertyId::CameraNear:
+	case lcObjectPropertyId::CameraFar:
+	case lcObjectPropertyId::CameraPositionX:
+	case lcObjectPropertyId::CameraPositionY:
+	case lcObjectPropertyId::CameraPositionZ:
+	case lcObjectPropertyId::CameraTargetX:
+	case lcObjectPropertyId::CameraTargetY:
+	case lcObjectPropertyId::CameraTargetZ:
+	case lcObjectPropertyId::CameraUpX:
+	case lcObjectPropertyId::CameraUpY:
+	case lcObjectPropertyId::CameraUpZ:
+	case lcObjectPropertyId::LightName:
+	case lcObjectPropertyId::LightType:
+	case lcObjectPropertyId::LightColor:
+	case lcObjectPropertyId::LightBlenderPower:
+	case lcObjectPropertyId::LightPOVRayPower:
+	case lcObjectPropertyId::LightCastShadow:
+	case lcObjectPropertyId::LightPOVRayFadeDistance:
+	case lcObjectPropertyId::LightPOVRayFadePower:
+	case lcObjectPropertyId::LightBlenderRadius:
+	case lcObjectPropertyId::LightBlenderAngle:
+	case lcObjectPropertyId::LightAreaSizeX:
+	case lcObjectPropertyId::LightAreaSizeY:
+	case lcObjectPropertyId::LightSpotConeAngle:
+	case lcObjectPropertyId::LightSpotPenumbraAngle:
+	case lcObjectPropertyId::LightPOVRaySpotTightness:
+	case lcObjectPropertyId::LightAreaShape:
+	case lcObjectPropertyId::LightPOVRayAreaGridX:
+	case lcObjectPropertyId::LightPOVRayAreaGridY:
+		return false;
+
+	case lcObjectPropertyId::ObjectPositionX:
+	case lcObjectPropertyId::ObjectPositionY:
+	case lcObjectPropertyId::ObjectPositionZ:
+		return mPosition.SetKeyFrame(Time, KeyFrame);
+
+	case lcObjectPropertyId::ObjectRotationX:
+	case lcObjectPropertyId::ObjectRotationY:
+	case lcObjectPropertyId::ObjectRotationZ:
+		return mRotation.SetKeyFrame(Time, KeyFrame);
+
+	case lcObjectPropertyId::Count:
+		return false;
+	}
+
+	return false;
 }
 
 void lcPiece::RemoveKeyFrames()
 {
-	mPositionKeys.RemoveAll();
-	mPositionKeys.ChangeKey(mModelWorld.GetTranslation(), 1, true);
-
-	mRotationKeys.RemoveAll();
-	mRotationKeys.ChangeKey(lcMatrix33(mModelWorld), 1, true);
+	mPosition.RemoveAllKeys();
+	mRotation.RemoveAllKeys();
 }
 
 void lcPiece::AddMainModelRenderMeshes(lcScene* Scene, bool Highlight, bool Fade) const
@@ -753,11 +994,11 @@ void lcPiece::MoveSelected(lcStep Step, bool AddKey, const lcVector3& Distance)
 
 		mModelWorld.SetTranslation(Position);
 	}
-	else
+	else if (Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST)
 	{
-		const int ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+		const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
 
-		if (ControlPointIndex >= 0 && ControlPointIndex < mControlPoints.GetSize())
+		if (ControlPointIndex < mControlPoints.size())
 		{
 			const lcMatrix33 InverseWorldMatrix = lcMatrix33AffineInverse(lcMatrix33(mModelWorld));
 			lcMatrix44& Transform = mControlPoints[ControlPointIndex].Transform;
@@ -791,11 +1032,11 @@ void lcPiece::Rotate(lcStep Step, bool AddKey, const lcMatrix33& RotationMatrix,
 		SetPosition(Center + Distance, Step, AddKey);
 		SetRotation(NewLocalToWorldMatrix, Step, AddKey);
 	}
-	else
+	else if (Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST)
 	{
-		const int ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+		const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
 
-		if (ControlPointIndex >= 0 && ControlPointIndex < mControlPoints.GetSize())
+		if (ControlPointIndex < mControlPoints.size())
 		{
 			lcMatrix44& Transform = mControlPoints[ControlPointIndex].Transform;
 			const lcMatrix33 PieceWorldMatrix(mModelWorld);
@@ -835,12 +1076,10 @@ void lcPiece::RotatePivotPoint(const lcMatrix33& RotationMatrix)
 
 quint32 lcPiece::GetAllowedTransforms() const
 {
-	constexpr quint32 Move = LC_OBJECT_TRANSFORM_MOVE_X | LC_OBJECT_TRANSFORM_MOVE_Y | LC_OBJECT_TRANSFORM_MOVE_Z;
-	constexpr quint32 Rotate = LC_OBJECT_TRANSFORM_ROTATE_X | LC_OBJECT_TRANSFORM_ROTATE_Y | LC_OBJECT_TRANSFORM_ROTATE_Z;
 	const quint32 Section = GetFocusSection();
 
 	if (Section == LC_PIECE_SECTION_POSITION || Section == LC_PIECE_SECTION_INVALID)
-		return Move | Rotate;
+		return LC_OBJECT_TRANSFORM_MOVE_XYZ | LC_OBJECT_TRANSFORM_ROTATE_XYZ;
 
 	const lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
 
@@ -850,18 +1089,142 @@ quint32 lcPiece::GetAllowedTransforms() const
 			return LC_OBJECT_TRANSFORM_MOVE_Z;
 
 		if (SynthInfo->IsCurve())
-			return Move | Rotate | LC_OBJECT_TRANSFORM_SCALE_X;
+			return LC_OBJECT_TRANSFORM_MOVE_XYZ | LC_OBJECT_TRANSFORM_ROTATE_XYZ | LC_OBJECT_TRANSFORM_SCALE_X;
 
 		if (SynthInfo->IsNondirectional())
-			return Move;
+			return LC_OBJECT_TRANSFORM_MOVE_XYZ;
 	}
 
 	return 0;
 }
 
+lcVector3 lcPiece::GetSectionPosition(quint32 Section) const
+{
+	if (Section == LC_PIECE_SECTION_POSITION)
+	{
+		if (mPivotPointValid)
+			return lcMul(mPivotMatrix, mModelWorld).GetTranslation();
+		else
+			return mModelWorld.GetTranslation();
+	}
+
+	if (mPieceInfo->GetSynthInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST)
+		{
+			const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+
+			if (ControlPointIndex < mControlPoints.size())
+			{
+				const lcMatrix44& Transform = mControlPoints[ControlPointIndex].Transform;
+				return lcMul(Transform, mModelWorld).GetTranslation();
+			}
+		}
+	}
+
+	if (mPieceInfo->GetTrainTrackInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST)
+		{
+			const quint32 ConnectionIndex = Section - LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST;
+			const std::vector<lcTrainTrackConnection>& Connections = mPieceInfo->GetTrainTrackInfo()->GetConnections();
+
+			if (ConnectionIndex < Connections.size())
+			{
+				const lcMatrix44& Transform = Connections[ConnectionIndex].Transform;
+				return lcMul(Transform, mModelWorld).GetTranslation();
+			}
+		}
+	}
+
+	return lcVector3(0.0f, 0.0f, 0.0f);
+}
+
+lcVector3 lcPiece::GetRotationCenter() const
+{
+	const quint32 Section = GetFocusSection();
+
+	if (Section == LC_PIECE_SECTION_POSITION || Section == LC_PIECE_SECTION_INVALID)
+	{
+		if (mPivotPointValid)
+			return lcMul31(mPivotMatrix.GetTranslation(), mModelWorld);
+	}
+	else if (mPieceInfo->GetSynthInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST)
+		{
+			const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+
+			if (ControlPointIndex < mControlPoints.size())
+			{
+				const lcMatrix44& Transform = mControlPoints[ControlPointIndex].Transform;
+				return lcMul31(Transform.GetTranslation(), mModelWorld);
+			}
+		}
+	}
+	else if (mPieceInfo->GetTrainTrackInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST)
+		{
+			const quint32 ConnectionIndex = Section - LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST;
+			const std::vector<lcTrainTrackConnection>& Connections = mPieceInfo->GetTrainTrackInfo()->GetConnections();
+
+			if (ConnectionIndex < Connections.size())
+			{
+				const lcMatrix44& Transform = Connections[ConnectionIndex].Transform;
+				return lcMul(Transform, mModelWorld).GetTranslation();
+			}
+		}
+	}
+
+	return mModelWorld.GetTranslation();
+}
+
+lcMatrix33 lcPiece::GetRelativeRotation() const
+{
+	const quint32 Section = GetFocusSection();
+
+	if (Section == LC_PIECE_SECTION_POSITION || Section == LC_PIECE_SECTION_INVALID)
+	{
+		if (mPivotPointValid)
+			return lcMatrix33(lcMul(mModelWorld, mPivotMatrix));
+		else
+			return lcMatrix33(mModelWorld);
+	}
+	else if (mPieceInfo->GetSynthInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST)
+		{
+			const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+
+			if (ControlPointIndex < mControlPoints.size())
+			{
+				const lcMatrix44& Transform = mControlPoints[ControlPointIndex].Transform;
+				return lcMatrix33(lcMul(Transform, mModelWorld));
+			}
+		}
+	}
+	else if (mPieceInfo->GetTrainTrackInfo())
+	{
+		if (Section >= LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST)
+		{
+			const quint32 ConnectionIndex = Section - LC_PIECE_SECTION_TRAIN_TRACK_CONNECTION_FIRST;
+			const std::vector<lcTrainTrackConnection>& Connections = mPieceInfo->GetTrainTrackInfo()->GetConnections();
+
+			if (ConnectionIndex < Connections.size())
+			{
+				const lcMatrix44& Transform = Connections[ConnectionIndex].Transform;
+				return lcMatrix33(lcMul(Transform, mModelWorld));
+			}
+		}
+	}
+
+	return lcMatrix33Identity();
+}
+
 bool lcPiece::CanAddControlPoint() const
 {
-	if (mControlPoints.GetSize() >= LC_MAX_CONTROL_POINTS)
+	if (mControlPoints.size() >= LC_MAX_CONTROL_POINTS)
 		return false;
 
 	const lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
@@ -871,7 +1234,7 @@ bool lcPiece::CanAddControlPoint() const
 bool lcPiece::CanRemoveControlPoint() const
 {
 	const quint32 Section = GetFocusSection();
-	return Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST && Section <= LC_PIECE_SECTION_CONTROL_POINT_LAST && mControlPoints.GetSize() > 2;
+	return Section >= LC_PIECE_SECTION_CONTROL_POINT_FIRST && Section <= LC_PIECE_SECTION_CONTROL_POINT_LAST && mControlPoints.size() > 2;
 }
 
 bool lcPiece::InsertControlPoint(const lcVector3& WorldStart, const lcVector3& WorldEnd)
@@ -899,32 +1262,37 @@ bool lcPiece::InsertControlPoint(const lcVector3& WorldStart, const lcVector3& W
 
 bool lcPiece::RemoveFocusedControlPoint()
 {
-	const int ControlPointIndex = GetFocusSection() - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+	quint32 Section = GetFocusSection();
 
-	if (ControlPointIndex < 0 || ControlPointIndex >= mControlPoints.GetSize() || mControlPoints.GetSize() <= 2)
+	if (Section < LC_PIECE_SECTION_CONTROL_POINT_FIRST)
+		return false;
+
+	const quint32 ControlPointIndex = Section - LC_PIECE_SECTION_CONTROL_POINT_FIRST;
+
+	if (ControlPointIndex >= mControlPoints.size() || mControlPoints.size() <= 2)
 		return false;
 
 	SetFocused(GetFocusSection(), false);
 	SetFocused(LC_PIECE_SECTION_POSITION, true);
-	mControlPoints.RemoveIndex(ControlPointIndex);
+	mControlPoints.erase(mControlPoints.begin() + ControlPointIndex);
 
 	UpdateMesh();
 
 	return true;
 }
 
-void lcPiece::VerifyControlPoints(lcArray<lcPieceControlPoint>& ControlPoints) const
+void lcPiece::VerifyControlPoints(std::vector<lcPieceControlPoint>& ControlPoints) const
 {
 	const lcSynthInfo* SynthInfo = mPieceInfo->GetSynthInfo();
 
 	if (!SynthInfo)
 	{
-		ControlPoints.RemoveAll();
+		ControlPoints.clear();
 	}
 	else
 	{
-		if (ControlPoints.GetSize() > LC_MAX_CONTROL_POINTS)
-			ControlPoints.SetSize(LC_MAX_CONTROL_POINTS);
+		if (ControlPoints.size() > LC_MAX_CONTROL_POINTS)
+			ControlPoints.resize(LC_MAX_CONTROL_POINTS);
 
 		SynthInfo->VerifyControlPoints(ControlPoints);
 	}
@@ -996,10 +1364,10 @@ lcGroup* lcPiece::GetTopGroup()
 
 void lcPiece::UpdatePosition(lcStep Step)
 {
-	const lcVector3 Position = mPositionKeys.CalculateKey(Step);
-	const lcMatrix33 Rotation = mRotationKeys.CalculateKey(Step);
+	mPosition.Update(Step);
+	mRotation.Update(Step);
 
-	mModelWorld = lcMatrix44(Rotation, Position);
+	mModelWorld = lcMatrix44(mRotation, mPosition);
 }
 
 void lcPiece::UpdateMesh()
