@@ -2,7 +2,6 @@
 
 #include <stdio.h>
 #include "lc_math.h"
-#include "lc_array.h"
 
 enum class lcPieceInfoType
 {
@@ -30,6 +29,7 @@ struct lcModelPartsEntry
 };
 
 class lcSynthInfo;
+class lcTrainTrackInfo;
 enum class lcZipFileType;
 
 class PieceInfo
@@ -64,6 +64,16 @@ public:
 		mSynthInfo = SynthInfo;
 	}
 
+	lcTrainTrackInfo* GetTrainTrackInfo() const
+	{
+		return mTrainTrackInfo;
+	}
+
+	void SetTrainTrackInfo(lcTrainTrackInfo* TrainTrackInfo)
+	{
+		mTrainTrackInfo = TrainTrackInfo;
+	}
+
 	lcMesh* GetMesh() const
 	{
 		return mMesh;
@@ -183,7 +193,7 @@ public:
 
 public:
 	char mFileName[LC_PIECE_NAME_LEN];
-	char m_strDescription[128];
+	char m_strDescription[256];
 	lcZipFileType mZipFileType;
 	int mZipFileIndex;
 	lcPieceInfoState mState;
@@ -193,12 +203,13 @@ public:
 protected:
 	void ReleaseMesh();
 
-	int mRefCount;
-	lcPieceInfoType mType;
-	lcModel* mModel;
-	Project* mProject;
-	lcMesh* mMesh;
+	int mRefCount = 0;
+	lcPieceInfoType mType = lcPieceInfoType::Part;
+	lcModel* mModel = nullptr;
+	Project* mProject = nullptr;
+	lcMesh* mMesh = nullptr;
 	lcBoundingBox mBoundingBox;
-	lcSynthInfo* mSynthInfo;
+	lcSynthInfo* mSynthInfo = nullptr;
+	lcTrainTrackInfo* mTrainTrackInfo = nullptr;
 };
 
