@@ -1,6 +1,5 @@
 #pragma once
 
-#include "lc_array.h"
 #include "lc_application.h"
 
 #define LC_HTML_SINGLEPAGE    0x01
@@ -39,7 +38,7 @@ public:
 	Project& operator=(const Project&) = delete;
 	Project& operator=(Project&&) = delete;
 
-	const lcArray<lcModel*>& GetModels() const
+	const std::vector<std::unique_ptr<lcModel>>& GetModels() const
 	{
 		return mModels;
 	}
@@ -51,14 +50,9 @@ public:
 		return mActiveModel;
 	}
 
-	int GetActiveModelIndex() const
-	{
-		return mModels.FindIndex(mActiveModel);
-	}
-
 	lcModel* GetMainModel() const
 	{
-		return !mModels.IsEmpty() ? mModels[0] : nullptr;
+		return !mModels.empty() ? mModels[0].get() : nullptr;
 	}
 
 	bool IsModified() const;
@@ -74,8 +68,9 @@ public:
 
 	lcInstructions* GetInstructions();
 
-	void SetActiveModel(int ModelIndex);
-	void SetActiveModel(const QString& FileName);
+	void SetActiveModel(lcModel* Model, bool UpdateInterface);
+	void SetActiveModel(int ModelIndex, bool UpdateInterface);
+	void SetActiveModel(const QString& FileName, bool UpdateInterface);
 
 	lcModel* CreateNewModel(bool ShowModel);
 	QString GetNewModelName(QWidget* ParentWidget, const QString& DialogTitle, const QString& CurrentName, const QStringList& ExistingModels) const;
@@ -89,13 +84,14 @@ public:
 	bool ImportInventory(const QByteArray& Inventory, const QString& Name, const QString& Description);
 
 	void SaveImage();
+	bool ExportCurrentStep(const QString& FileName);
 	bool ExportModel(const QString& FileName, lcModel* Model) const;
 	bool Export3DStudio(const QString& FileName);
 	void ExportBrickLink();
 	bool ExportCOLLADA(const QString& FileName);
 	bool ExportCSV(const QString& FileName);
 	void ExportHTML(const lcHTMLExportOptions& Options);
-	bool ExportPOVRay(const QString& FileName);
+	std::pair<bool, QString> ExportPOVRay(const QString& FileName);
 	bool ExportWavefront(const QString& FileName);
 
 	void UpdatePieceInfo(PieceInfo* Info) const;
@@ -111,7 +107,7 @@ protected:
 	QString mFileName;
 	QFileSystemWatcher mFileWatcher;
 
-	lcArray<lcModel*> mModels;
+	std::vector<std::unique_ptr<lcModel>> mModels;
 	lcModel* mActiveModel;
 	std::unique_ptr<lcInstructions> mInstructions;
 
