@@ -2,13 +2,14 @@
 #include "lc_qpreferencesdialog.h"
 #include "ui_lc_qpreferencesdialog.h"
 #include "lc_qutils.h"
-#include "lc_qcategorydialog.h"
+#include "lc_categorydialog.h"
 #include "lc_library.h"
 #include "lc_application.h"
 #include "lc_qutils.h"
 #include "lc_glextensions.h"
 #include "pieceinf.h"
 #include "lc_edgecolordialog.h"
+#include "lc_blenderpreferences.h"
 
 static const char* gLanguageLocales[] =
 {
@@ -45,12 +46,18 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget* Parent, lcPreferencesDialogO
 	connect(ui->ViewSphereColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
 	connect(ui->ViewSphereTextColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
 	connect(ui->ViewSphereHighlightColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
-	connect(ui->categoriesTree, SIGNAL(itemSelectionChanged()), this, SLOT(updateParts()));
+	connect(ui->ObjectSelectedColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->ObjectFocusedColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->CameraColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->LightColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->ControlPointColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->ControlPointFocusedColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::ColorButtonClicked);
+	connect(ui->categoriesTree, &QTreeWidget::itemSelectionChanged, this, &lcQPreferencesDialog::updateParts);
 	ui->shortcutEdit->installEventFilter(this);
-	connect(ui->commandList, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(commandChanged(QTreeWidgetItem*)));
-	connect(ui->mouseTree, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(MouseTreeItemChanged(QTreeWidgetItem*)));
-	connect(ui->HighContrastButton, SIGNAL(clicked()), this, SLOT(AutomateEdgeColor()));
-	connect(ui->AutomateEdgeColorButton, SIGNAL(clicked()), this, SLOT(AutomateEdgeColor()));
+	connect(ui->commandList, &QTreeWidget::currentItemChanged, this, &lcQPreferencesDialog::commandChanged);
+	connect(ui->mouseTree, &QTreeWidget::currentItemChanged, this, &lcQPreferencesDialog::MouseTreeItemChanged);
+	connect(ui->HighContrastButton, &QToolButton::clicked, this, &lcQPreferencesDialog::AutomateEdgeColor);
+	connect(ui->AutomateEdgeColorButton, &QToolButton::clicked, this, &lcQPreferencesDialog::AutomateEdgeColor);
 
 	ui->partsLibrary->setText(mOptions->LibraryPath);
 	ui->ColorConfigEdit->setText(mOptions->ColorConfigPath);
@@ -105,6 +112,9 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget* Parent, lcPreferencesDialogO
 		mLineWidthGranularity = 1.0f;
 	}
 
+	mLineWidthRange[1] = qMin(mLineWidthRange[1], 10.0f);
+	mLineWidthGranularity = qMax(mLineWidthGranularity, 0.25f);
+
 	ui->LineWidthSlider->setRange(0, (mLineWidthRange[1] - mLineWidthRange[0]) / mLineWidthGranularity);
 	ui->LineWidthSlider->setValue((mOptions->Preferences.mLineWidth - mLineWidthRange[0]) / mLineWidthGranularity);
 
@@ -210,6 +220,12 @@ lcQPreferencesDialog::lcQPreferencesDialog(QWidget* Parent, lcPreferencesDialogO
 	SetButtonPixmap(mOptions->Preferences.mViewSphereColor, ui->ViewSphereColorButton);
 	SetButtonPixmap(mOptions->Preferences.mViewSphereTextColor, ui->ViewSphereTextColorButton);
 	SetButtonPixmap(mOptions->Preferences.mViewSphereHighlightColor, ui->ViewSphereHighlightColorButton);
+	SetButtonPixmap(mOptions->Preferences.mObjectSelectedColor, ui->ObjectSelectedColorButton);
+	SetButtonPixmap(mOptions->Preferences.mObjectFocusedColor, ui->ObjectFocusedColorButton);
+	SetButtonPixmap(mOptions->Preferences.mCameraColor, ui->CameraColorButton);
+	SetButtonPixmap(mOptions->Preferences.mLightColor, ui->LightColorButton);
+	SetButtonPixmap(mOptions->Preferences.mControlPointColor, ui->ControlPointColorButton);
+	SetButtonPixmap(mOptions->Preferences.mControlPointFocusedColor, ui->ControlPointFocusedColorButton);
 
 	on_studStyleCombo_currentIndexChanged(ui->studStyleCombo->currentIndex());
 	on_antiAliasing_toggled();
@@ -353,6 +369,23 @@ void lcQPreferencesDialog::on_partsArchiveBrowse_clicked()
 		ui->partsLibrary->setText(QDir::toNativeSeparators(result));
 }
 
+void lcQPreferencesDialog::on_BlenderAddonSettingsButton_clicked()
+{
+	int Width = 1280;
+	int Height = 720;
+	double Scale = 1.0f;
+
+	lcBlenderPreferencesDialog::GetBlenderPreferences(
+		Width,
+		Height,
+		Scale,
+		this);
+
+	Q_UNUSED(Width)
+	Q_UNUSED(Height)
+	Q_UNUSED(Scale)
+}
+
 void lcQPreferencesDialog::on_ColorConfigBrowseButton_clicked()
 {
 	QString Result = QFileDialog::getOpenFileName(this, tr("Select Color Configuration File"), ui->ColorConfigEdit->text(), tr("Settings Files (*.ldr);;All Files (*.*)"));
@@ -495,6 +528,38 @@ void lcQPreferencesDialog::ColorButtonClicked()
 		Color = &mOptions->Preferences.mViewSphereHighlightColor;
 		Title = tr("Select View Sphere Highlight Color");
 	}
+	else if (Button == ui->ObjectSelectedColorButton)
+	{
+		Color = &mOptions->Preferences.mObjectSelectedColor;
+		Title = tr("Select Object Selected Color");
+	}
+	else if (Button == ui->ObjectFocusedColorButton)
+	{
+		Color = &mOptions->Preferences.mObjectFocusedColor;
+		Title = tr("Select Object Focused Color");
+	}
+	else if (Button == ui->CameraColorButton)
+	{
+		Color = &mOptions->Preferences.mCameraColor;
+		Title = tr("Select Camera Color");
+	}
+	else if (Button == ui->LightColorButton)
+	{
+		Color = &mOptions->Preferences.mLightColor;
+		Title = tr("Select Light Color");
+	}
+	else if (Button == ui->ControlPointColorButton)
+	{
+		Color = &mOptions->Preferences.mControlPointColor;
+		Title = tr("Select Control Point Color");
+		DialogOptions = QColorDialog::ShowAlphaChannel;
+	}
+	else if (Button == ui->ControlPointFocusedColorButton)
+	{
+		Color = &mOptions->Preferences.mControlPointFocusedColor;
+		Title = tr("Select Control Point Focused Color");
+		DialogOptions = QColorDialog::ShowAlphaChannel;
+	}
 	else
 		return;
 
@@ -592,9 +657,13 @@ void lcQPreferencesDialog::AutomateEdgeColor()
 	lcAutomateEdgeColorDialog Dialog(this, sender() == ui->HighContrastButton);
 	if (Dialog.exec() == QDialog::Accepted)
 	{
+		mOptions->Preferences.mStudCylinderColorEnabled = Dialog.mStudCylinderColorEnabled;
 		mOptions->Preferences.mStudCylinderColor = Dialog.mStudCylinderColor;
+		mOptions->Preferences.mPartEdgeColorEnabled = Dialog.mPartEdgeColorEnabled;
 		mOptions->Preferences.mPartEdgeColor = Dialog.mPartEdgeColor;
+		mOptions->Preferences.mBlackEdgeColorEnabled = Dialog.mBlackEdgeColorEnabled;
 		mOptions->Preferences.mBlackEdgeColor = Dialog.mBlackEdgeColor;
+		mOptions->Preferences.mDarkEdgeColorEnabled = Dialog.mDarkEdgeColorEnabled;
 		mOptions->Preferences.mDarkEdgeColor = Dialog.mDarkEdgeColor;
 		mOptions->Preferences.mPartEdgeContrast = Dialog.mPartEdgeContrast;
 		mOptions->Preferences.mPartColorValueLDIndex = Dialog.mPartColorValueLDIndex;
@@ -635,16 +704,14 @@ void lcQPreferencesDialog::updateParts()
 
 	if (categoryIndex != -1)
 	{
-		lcArray<PieceInfo*> singleParts, groupedParts;
+		std::vector<PieceInfo*> SingleParts, GroupedParts;
 
-		Library->GetCategoryEntries(mOptions->Categories[categoryIndex].Keywords.constData(), false, singleParts, groupedParts);
+		Library->GetCategoryEntries(mOptions->Categories[categoryIndex].Keywords.constData(), false, SingleParts, GroupedParts);
 
-		for (int partIndex = 0; partIndex < singleParts.GetSize(); partIndex++)
+		for (PieceInfo* Info : SingleParts)
 		{
-			PieceInfo *info = singleParts[partIndex];
-
-			QStringList rowList(info->m_strDescription);
-			rowList.append(info->mFileName);
+			QStringList rowList(Info->m_strDescription);
+			rowList.append(Info->mFileName);
 
 			new QTreeWidgetItem(tree, rowList);
 		}
@@ -679,7 +746,7 @@ void lcQPreferencesDialog::on_newCategory_clicked()
 {
 	lcLibraryCategory category;
 
-	lcQCategoryDialog dialog(this, &category);
+	lcCategoryDialog dialog(this, &category);
 	if (dialog.exec() != QDialog::Accepted)
 		return;
 
@@ -704,7 +771,7 @@ void lcQPreferencesDialog::on_editCategory_clicked()
 	if (categoryIndex == -1)
 		return;
 
-	lcQCategoryDialog dialog(this, &mOptions->Categories[categoryIndex]);
+	lcCategoryDialog dialog(this, &mOptions->Categories[categoryIndex]);
 	if (dialog.exec() != QDialog::Accepted)
 		return;
 
@@ -938,7 +1005,7 @@ void lcQPreferencesDialog::on_KeyboardFilterEdit_textEdited(const QString& Text)
 	{
 		std::function<bool(QTreeWidgetItem*,bool)> ShowItems = [&ShowItems, &Text](QTreeWidgetItem* ParentItem, bool ForceVisible)
 		{
-			ForceVisible |= (bool)ParentItem->text(0).contains(Text, Qt::CaseInsensitive) | (bool)ParentItem->text(1).contains(Text, Qt::CaseInsensitive);
+			ForceVisible |= ParentItem->text(0).contains(Text, Qt::CaseInsensitive) || ParentItem->text(1).contains(Text, Qt::CaseInsensitive);
 			bool Visible = ForceVisible;
 
 			for (int ChildIdx = 0; ChildIdx < ParentItem->childCount(); ChildIdx++)
